"""General-use classes to interact with the EMR service through CloudFormation.

See Also:
    `AWS developer guide for EMR
    <https://docs.aws.amazon.com/dms/latest/userguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import emr as _raw

# noinspection PyUnresolvedReferences
from .._raw.emr import *



"""General-use classes to interact with the ElasticLoadBalancing service through CloudFormation.

See Also:
    `AWS developer guide for ElasticLoadBalancing
    <https://docs.aws.amazon.com/elasticloadbalancing/latest/userguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import elasticloadbalancing as _raw

# noinspection PyUnresolvedReferences
from .._raw.elasticloadbalancing import *



"""Raw representations of every data type in the AWS Elasticsearch service.

See Also:
    `AWS developer guide for Elasticsearch
    <https://docs.aws.amazon.com/workspaces/latest/adminguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Domain",
    "DomainProperties",
]


@attrs(**ATTRSCONFIG)
class DomainProperties(ResourceProperties):
    AccessPolicies = attrib(default=None)
    AdvancedOptions = attrib(default=None)
    DomainName = attrib(default=None)
    EBSOptions = attrib(default=None)
    ElasticsearchClusterConfig = attrib(default=None)
    ElasticsearchVersion = attrib(default=None)
    EncryptionAtRestOptions = attrib(default=None)
    NodeToNodeEncryptionOptions = attrib(default=None)
    SnapshotOptions = attrib(default=None)
    Tags = attrib(default=None)
    VPCOptions = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Domain(Resource):
    """A Domain for Elasticsearch.

    See Also:
        `AWS Cloud Formation documentation for Domain
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticsearch-domain.html>`_
    """

    RESOURCE_TYPE = "AWS::Elasticsearch::Domain"

    Properties: DomainProperties = attrib(
        factory=DomainProperties,
        converter=create_object_converter(DomainProperties),
    )



"""General-use classes to interact with the EC2 service through CloudFormation.

See Also:
    `AWS developer guide for EC2
    <http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/>`_
"""

from attr import attrib
from attr import attrs

from flyingcircus.core import ATTRSCONFIG
from flyingcircus.core import Stack
from flyingcircus.intrinsic_function import Ref
from .._raw import ec2 as _raw
from .._raw.ec2 import *


class SecurityGroup(_raw.SecurityGroup):
    __slots__ = []

    #: Empty SecurityGroupEgressRule that can be used to replace the default.
    #:
    #: When Cloud Formation creates a security group, if there are no egress
    #: rules specified then it always creates a default egress rule that
    #: allows all-traffic. Sometimes this is undesirable, so the approved
    #: workaround is to create an egress rule that has no effect (eg. allows
    #: traffic to localhost).
    #:
    #:  See Also:
    #:      `Example of replacing the default rule in the SecurityGroup documentation
    #:      <https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group.html>`_
    EMPTY_EGRESS_RULE = dict(
        CidrIp="127.0.0.1/32",
        Description="NOOP rule to prevent CloudFormation from adding a rule that allows all traffic",
        IpProtocol="-1",
    )


@attrs(**ATTRSCONFIG)
class VPC(_raw.VPC):
    _internet_gateway: InternetGateway = attrib(default=None)
    _internet_gateway_attachment: VPCGatewayAttachment = attrib(default=None)

    @property
    def internet_gateway(self) -> InternetGateway:
        """Get the Internet Gateway associated with this VPC.

        This will only work if the internet gateway was created using
        `ensure_internet_gateway_exists`.
        """
        assert self._internet_gateway is not None, "`ensure_internet_gateway_exists` should be called first"
        return self._internet_gateway

    @property
    def internet_gateway_attachment(self) -> VPCGatewayAttachment:
        """Get the attachment object for the Internet Gateway associated with this VPC.

        This will only work if the internet gateway was created using
        `ensure_internet_gateway_exists`.
        """
        assert self._internet_gateway_attachment is not None, "`ensure_internet_gateway_exists` should be called first"
        return self._internet_gateway_attachment

    def ensure_internet_gateway_exists(self, stack: Stack):
        """Ensure there is an internet gateway attached to this VPC as part of the stack."""
        # If we have already attached a gateway to this VPC, then there is
        # nothing more to do
        if self._internet_gateway is not None:
            # Check that the internet gateway is in the desired stack
            if not [res for res in stack.Resources.values() if res is self._internet_gateway]:
                raise RuntimeError("Existing InternetGateway is not in this stack")
            if not [res for res in stack.Resources.values() if res is self._internet_gateway_attachment]:
                raise RuntimeError("Existing VPCGatewayAttachment for InternetGateway is not in this stack")

            return

        # Look for an existing gateway attached to this VPC
        for res in stack.Resources.values():
            if isinstance(res, VPCGatewayAttachment) \
                    and res.Properties.VpcId == Ref(self) \
                    and res.Properties.InternetGatewayId:
                self._internet_gateway_attachment = res

                # Try to dodgily unwrap the internet gateway...
                gateway_ref = res.Properties.InternetGatewayId
                if not isinstance(gateway_ref, Ref):
                    raise RuntimeError("Can't deal with direct ID references!")
                if not isinstance(gateway_ref._data, InternetGateway):
                    raise RuntimeError("There's something weird attached to this VPC instead of an Internet Gateway")
                self._internet_gateway = gateway_ref._data

                return

        # Create an Internet Gateway
        self._internet_gateway = InternetGateway()
        if self.name:
            # Copy the Name of the VPC if one has already been set
            self._internet_gateway.name = self.name

        igw_stack_name = stack.get_logical_name(self, resources_only=True) + "IGW"
        if igw_stack_name in stack.Resources:
            raise RuntimeError(f"There's already a resource named {igw_stack_name}")
        stack.Resources[igw_stack_name] = self._internet_gateway

        # Attach the gateway to this VPC
        self._internet_gateway_attachment = VPCGatewayAttachment(
            Properties=VPCGatewayAttachmentProperties(
                InternetGatewayId=Ref(self._internet_gateway),
                VpcId=Ref(self),
            ),
        )

        attachment_stack_name = igw_stack_name + "Attachment"
        if attachment_stack_name in stack.Resources:
            raise RuntimeError(f"There's already a resource named {attachment_stack_name}")
        stack.Resources[attachment_stack_name] = self._internet_gateway_attachment



"""Raw representations of every data type in the AWS Budgets service.

See Also:
    `AWS developer guide for Budgets
    <https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Budget",
    "BudgetProperties",
]


@attrs(**ATTRSCONFIG)
class BudgetProperties(ResourceProperties):
    Budget = attrib(default=None)
    NotificationsWithSubscribers = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Budget(Resource):
    """A Budget for Budgets.

    See Also:
        `AWS Cloud Formation documentation for Budget
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-budgets-budget.html>`_
    """

    RESOURCE_TYPE = "AWS::Budgets::Budget"

    Properties: BudgetProperties = attrib(
        factory=BudgetProperties,
        converter=create_object_converter(BudgetProperties),
    )



"""General-use classes to interact with the CertificateManager service through CloudFormation.

See Also:
    `AWS developer guide for CertificateManager
    <https://docs.aws.amazon.com/acm/latest/userguide/acm-overview.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import certificatemanager as _raw

# noinspection PyUnresolvedReferences
from .._raw.certificatemanager import *



"""General-use classes to interact with the AppStream service through CloudFormation.

See Also:
    `AWS developer guide for AppStream
    <https://docs.aws.amazon.com/appstream2/latest/developerguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import appstream as _raw

# noinspection PyUnresolvedReferences
from .._raw.appstream import *



"""General-use classes to interact with the DLM service through CloudFormation.

See Also:
    `AWS developer guide for DLM
    <https://docs.aws.amazon.com/dlm/latest/APIReference/Welcome.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import dlm as _raw

# noinspection PyUnresolvedReferences
from .._raw.dlm import *



"""General-use classes to interact with the SES service through CloudFormation.

See Also:
    `AWS developer guide for SES
    <https://docs.aws.amazon.com/servicecatalog/latest/dg/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import ses as _raw

# noinspection PyUnresolvedReferences
from .._raw.ses import *



"""General-use classes to interact with the IoT1Click service through CloudFormation.

See Also:
    `AWS developer guide for IoT1Click
    <https://docs.aws.amazon.com/iot-1-click/latest/developerguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import iot1click as _raw

# noinspection PyUnresolvedReferences
from .._raw.iot1click import *



"""Raw representations of every data type in the AWS Redshift service.

See Also:
    `AWS developer guide for Redshift
    <https://docs.aws.amazon.com/redshift/latest/gsg/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Cluster",
    "ClusterProperties",
    "ClusterParameterGroup",
    "ClusterParameterGroupProperties",
    "ClusterSecurityGroup",
    "ClusterSecurityGroupProperties",
    "ClusterSecurityGroupIngress",
    "ClusterSecurityGroupIngressProperties",
    "ClusterSubnetGroup",
    "ClusterSubnetGroupProperties",
]


@attrs(**ATTRSCONFIG)
class ClusterProperties(ResourceProperties):
    AllowVersionUpgrade = attrib(default=None)
    AutomatedSnapshotRetentionPeriod = attrib(default=None)
    AvailabilityZone = attrib(default=None)
    ClusterIdentifier = attrib(default=None)
    ClusterParameterGroupName = attrib(default=None)
    ClusterSecurityGroups = attrib(default=None)
    ClusterSubnetGroupName = attrib(default=None)
    ClusterType = attrib(default=None)
    ClusterVersion = attrib(default=None)
    DBName = attrib(default=None)
    ElasticIp = attrib(default=None)
    Encrypted = attrib(default=None)
    HsmClientCertificateIdentifier = attrib(default=None)
    HsmConfigurationIdentifier = attrib(default=None)
    IamRoles = attrib(default=None)
    KmsKeyId = attrib(default=None)
    LoggingProperties = attrib(default=None)
    MasterUsername = attrib(default=None)
    MasterUserPassword = attrib(default=None)
    NodeType = attrib(default=None)
    NumberOfNodes = attrib(default=None)
    OwnerAccount = attrib(default=None)
    Port = attrib(default=None)
    PreferredMaintenanceWindow = attrib(default=None)
    PubliclyAccessible = attrib(default=None)
    SnapshotClusterIdentifier = attrib(default=None)
    SnapshotIdentifier = attrib(default=None)
    Tags = attrib(default=None)
    VpcSecurityGroupIds = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Cluster(Resource):
    """A Cluster for Redshift.

    See Also:
        `AWS Cloud Formation documentation for Cluster
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html>`_
    """

    RESOURCE_TYPE = "AWS::Redshift::Cluster"

    Properties: ClusterProperties = attrib(
        factory=ClusterProperties,
        converter=create_object_converter(ClusterProperties),
    )


@attrs(**ATTRSCONFIG)
class ClusterParameterGroupProperties(ResourceProperties):
    Description = attrib(default=None)
    ParameterGroupFamily = attrib(default=None)
    Parameters = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ClusterParameterGroup(Resource):
    """A Cluster Parameter Group for Redshift.

    See Also:
        `AWS Cloud Formation documentation for ClusterParameterGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clusterparametergroup.html>`_
    """

    RESOURCE_TYPE = "AWS::Redshift::ClusterParameterGroup"

    Properties: ClusterParameterGroupProperties = attrib(
        factory=ClusterParameterGroupProperties,
        converter=create_object_converter(ClusterParameterGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class ClusterSecurityGroupProperties(ResourceProperties):
    Description = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ClusterSecurityGroup(Resource):
    """A Cluster Security Group for Redshift.

    See Also:
        `AWS Cloud Formation documentation for ClusterSecurityGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersecuritygroup.html>`_
    """

    RESOURCE_TYPE = "AWS::Redshift::ClusterSecurityGroup"

    Properties: ClusterSecurityGroupProperties = attrib(
        factory=ClusterSecurityGroupProperties,
        converter=create_object_converter(ClusterSecurityGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class ClusterSecurityGroupIngressProperties(ResourceProperties):
    CIDRIP = attrib(default=None)
    ClusterSecurityGroupName = attrib(default=None)
    EC2SecurityGroupName = attrib(default=None)
    EC2SecurityGroupOwnerId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ClusterSecurityGroupIngress(Resource):
    """A Cluster Security Group Ingress for Redshift.

    See Also:
        `AWS Cloud Formation documentation for ClusterSecurityGroupIngress
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersecuritygroupingress.html>`_
    """

    RESOURCE_TYPE = "AWS::Redshift::ClusterSecurityGroupIngress"

    Properties: ClusterSecurityGroupIngressProperties = attrib(
        factory=ClusterSecurityGroupIngressProperties,
        converter=create_object_converter(ClusterSecurityGroupIngressProperties),
    )


@attrs(**ATTRSCONFIG)
class ClusterSubnetGroupProperties(ResourceProperties):
    Description = attrib(default=None)
    SubnetIds = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ClusterSubnetGroup(Resource):
    """A Cluster Subnet Group for Redshift.

    See Also:
        `AWS Cloud Formation documentation for ClusterSubnetGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersubnetgroup.html>`_
    """

    RESOURCE_TYPE = "AWS::Redshift::ClusterSubnetGroup"

    Properties: ClusterSubnetGroupProperties = attrib(
        factory=ClusterSubnetGroupProperties,
        converter=create_object_converter(ClusterSubnetGroupProperties),
    )



"""General-use classes to interact with the CloudFront service through CloudFormation.

See Also:
    `AWS developer guide for CloudFront
    <https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/Introduction.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import cloudfront as _raw

# noinspection PyUnresolvedReferences
from .._raw.cloudfront import *



"""General-use classes to interact with the Cloud9 service through CloudFormation.

See Also:
    `AWS developer guide for Cloud9
    <https://docs.aws.amazon.com/cloud9/latest/user-guide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import cloud9 as _raw

# noinspection PyUnresolvedReferences
from .._raw.cloud9 import *



"""Raw representations of every data type in the AWS Athena service.

See Also:
    `AWS developer guide for Athena
    <https://docs.aws.amazon.com/athena/latest/ug/what-is.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "NamedQuery",
    "NamedQueryProperties",
]


@attrs(**ATTRSCONFIG)
class NamedQueryProperties(ResourceProperties):
    Database = attrib(default=None)
    Description = attrib(default=None)
    Name = attrib(default=None)
    QueryString = attrib(default=None)


@attrs(**ATTRSCONFIG)
class NamedQuery(Resource):
    """A Named Query for Athena.

    See Also:
        `AWS Cloud Formation documentation for NamedQuery
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-athena-namedquery.html>`_
    """

    RESOURCE_TYPE = "AWS::Athena::NamedQuery"

    Properties: NamedQueryProperties = attrib(
        factory=NamedQueryProperties,
        converter=create_object_converter(NamedQueryProperties),
    )



"""General-use classes to interact with the EFS service through CloudFormation.

See Also:
    `AWS developer guide for EFS
    <https://docs.aws.amazon.com/efs/latest/ug/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import efs as _raw

# noinspection PyUnresolvedReferences
from .._raw.efs import *



"""General-use classes to interact with the ElastiCache service through CloudFormation.

See Also:
    `AWS developer guide for ElastiCache
    <https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import elasticache as _raw

# noinspection PyUnresolvedReferences
from .._raw.elasticache import *



"""General-use classes to interact with the ApplicationAutoScaling service through CloudFormation.

See Also:
    `AWS developer guide for ApplicationAutoScaling
    <https://docs.aws.amazon.com/autoscaling/application/APIReference/Welcome.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import applicationautoscaling as _raw

# noinspection PyUnresolvedReferences
from .._raw.applicationautoscaling import *



"""General-use classes to interact with the Route53 service through CloudFormation.

See Also:
    `AWS developer guide for Route53
    <https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import route53 as _raw

# noinspection PyUnresolvedReferences
from .._raw.route53 import *



"""General-use classes to interact with the EKS service through CloudFormation.

See Also:
    `AWS developer guide for EKS
    <https://docs.aws.amazon.com/eks/latest/userguide/what-is-eks.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import eks as _raw

# noinspection PyUnresolvedReferences
from .._raw.eks import *



"""General-use classes to interact with the RDS service through CloudFormation.

See Also:
    `AWS developer guide for RDS
    <https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/index.html>`_
"""

from .._raw import rds as _raw

# noinspection PyUnresolvedReferences
from .._raw.rds import *


class DBInstance(_raw.DBInstance):
    __slots__ = []

    # Implement Naming
    # ----------------
    @property
    def name(self):
        return self.Properties["DBInstanceIdentifier"]  # Might be default of None

    @name.setter
    def name(self, value: str):
        self.Properties["DBInstanceIdentifier"] = value



"""Raw representations of every data type in the AWS IoT service.

See Also:
    `AWS developer guide for IoT
    <https://docs.aws.amazon.com/iot/latest/developerguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Certificate",
    "CertificateProperties",
    "Policy",
    "PolicyProperties",
    "PolicyPrincipalAttachment",
    "PolicyPrincipalAttachmentProperties",
    "Thing",
    "ThingProperties",
    "ThingPrincipalAttachment",
    "ThingPrincipalAttachmentProperties",
    "TopicRule",
    "TopicRuleProperties",
]


@attrs(**ATTRSCONFIG)
class CertificateProperties(ResourceProperties):
    CertificateSigningRequest = attrib(default=None)
    Status = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Certificate(Resource):
    """A Certificate for IoT.

    See Also:
        `AWS Cloud Formation documentation for Certificate
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-certificate.html>`_
    """

    RESOURCE_TYPE = "AWS::IoT::Certificate"

    Properties: CertificateProperties = attrib(
        factory=CertificateProperties,
        converter=create_object_converter(CertificateProperties),
    )


@attrs(**ATTRSCONFIG)
class PolicyProperties(ResourceProperties):
    PolicyDocument = attrib(default=None)
    PolicyName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Policy(Resource):
    """A Policy for IoT.

    See Also:
        `AWS Cloud Formation documentation for Policy
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-policy.html>`_
    """

    RESOURCE_TYPE = "AWS::IoT::Policy"

    Properties: PolicyProperties = attrib(
        factory=PolicyProperties,
        converter=create_object_converter(PolicyProperties),
    )


@attrs(**ATTRSCONFIG)
class PolicyPrincipalAttachmentProperties(ResourceProperties):
    PolicyName = attrib(default=None)
    Principal = attrib(default=None)


@attrs(**ATTRSCONFIG)
class PolicyPrincipalAttachment(Resource):
    """A Policy Principal Attachment for IoT.

    See Also:
        `AWS Cloud Formation documentation for PolicyPrincipalAttachment
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-policyprincipalattachment.html>`_
    """

    RESOURCE_TYPE = "AWS::IoT::PolicyPrincipalAttachment"

    Properties: PolicyPrincipalAttachmentProperties = attrib(
        factory=PolicyPrincipalAttachmentProperties,
        converter=create_object_converter(PolicyPrincipalAttachmentProperties),
    )


@attrs(**ATTRSCONFIG)
class ThingProperties(ResourceProperties):
    AttributePayload = attrib(default=None)
    ThingName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Thing(Resource):
    """A Thing for IoT.

    See Also:
        `AWS Cloud Formation documentation for Thing
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-thing.html>`_
    """

    RESOURCE_TYPE = "AWS::IoT::Thing"

    Properties: ThingProperties = attrib(
        factory=ThingProperties,
        converter=create_object_converter(ThingProperties),
    )


@attrs(**ATTRSCONFIG)
class ThingPrincipalAttachmentProperties(ResourceProperties):
    Principal = attrib(default=None)
    ThingName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ThingPrincipalAttachment(Resource):
    """A Thing Principal Attachment for IoT.

    See Also:
        `AWS Cloud Formation documentation for ThingPrincipalAttachment
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-thingprincipalattachment.html>`_
    """

    RESOURCE_TYPE = "AWS::IoT::ThingPrincipalAttachment"

    Properties: ThingPrincipalAttachmentProperties = attrib(
        factory=ThingPrincipalAttachmentProperties,
        converter=create_object_converter(ThingPrincipalAttachmentProperties),
    )


@attrs(**ATTRSCONFIG)
class TopicRuleProperties(ResourceProperties):
    RuleName = attrib(default=None)
    TopicRulePayload = attrib(default=None)


@attrs(**ATTRSCONFIG)
class TopicRule(Resource):
    """A Topic Rule for IoT.

    See Also:
        `AWS Cloud Formation documentation for TopicRule
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-topicrule.html>`_
    """

    RESOURCE_TYPE = "AWS::IoT::TopicRule"

    Properties: TopicRuleProperties = attrib(
        factory=TopicRuleProperties,
        converter=create_object_converter(TopicRuleProperties),
    )



"""Raw representations of every data type in the AWS GuardDuty service.

See Also:
    `AWS developer guide for GuardDuty
    <https://docs.aws.amazon.com/guardduty/latest/ug/what-is-guardduty.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Detector",
    "DetectorProperties",
    "Filter",
    "FilterProperties",
    "IPSet",
    "IPSetProperties",
    "Master",
    "MasterProperties",
    "Member",
    "MemberProperties",
    "ThreatIntelSet",
    "ThreatIntelSetProperties",
]


@attrs(**ATTRSCONFIG)
class DetectorProperties(ResourceProperties):
    Enable = attrib(default=None)
    FindingPublishingFrequency = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Detector(Resource):
    """A Detector for GuardDuty.

    See Also:
        `AWS Cloud Formation documentation for Detector
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-guardduty-detector.html>`_
    """

    RESOURCE_TYPE = "AWS::GuardDuty::Detector"

    Properties: DetectorProperties = attrib(
        factory=DetectorProperties,
        converter=create_object_converter(DetectorProperties),
    )


@attrs(**ATTRSCONFIG)
class FilterProperties(ResourceProperties):
    Action = attrib(default=None)
    Description = attrib(default=None)
    DetectorId = attrib(default=None)
    FindingCriteria = attrib(default=None)
    Name = attrib(default=None)
    Rank = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Filter(Resource):
    """A Filter for GuardDuty.

    See Also:
        `AWS Cloud Formation documentation for Filter
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-guardduty-filter.html>`_
    """

    RESOURCE_TYPE = "AWS::GuardDuty::Filter"

    Properties: FilterProperties = attrib(
        factory=FilterProperties,
        converter=create_object_converter(FilterProperties),
    )


@attrs(**ATTRSCONFIG)
class IPSetProperties(ResourceProperties):
    Activate = attrib(default=None)
    DetectorId = attrib(default=None)
    Format = attrib(default=None)
    Location = attrib(default=None)
    Name = attrib(default=None)


@attrs(**ATTRSCONFIG)
class IPSet(Resource):
    """A Ip Set for GuardDuty.

    See Also:
        `AWS Cloud Formation documentation for IPSet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-guardduty-ipset.html>`_
    """

    RESOURCE_TYPE = "AWS::GuardDuty::IPSet"

    Properties: IPSetProperties = attrib(
        factory=IPSetProperties,
        converter=create_object_converter(IPSetProperties),
    )


@attrs(**ATTRSCONFIG)
class MasterProperties(ResourceProperties):
    DetectorId = attrib(default=None)
    InvitationId = attrib(default=None)
    MasterId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Master(Resource):
    """A Master for GuardDuty.

    See Also:
        `AWS Cloud Formation documentation for Master
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-guardduty-master.html>`_
    """

    RESOURCE_TYPE = "AWS::GuardDuty::Master"

    Properties: MasterProperties = attrib(
        factory=MasterProperties,
        converter=create_object_converter(MasterProperties),
    )


@attrs(**ATTRSCONFIG)
class MemberProperties(ResourceProperties):
    DetectorId = attrib(default=None)
    DisableEmailNotification = attrib(default=None)
    Email = attrib(default=None)
    MemberId = attrib(default=None)
    Message = attrib(default=None)
    Status = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Member(Resource):
    """A Member for GuardDuty.

    See Also:
        `AWS Cloud Formation documentation for Member
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-guardduty-member.html>`_
    """

    RESOURCE_TYPE = "AWS::GuardDuty::Member"

    Properties: MemberProperties = attrib(
        factory=MemberProperties,
        converter=create_object_converter(MemberProperties),
    )


@attrs(**ATTRSCONFIG)
class ThreatIntelSetProperties(ResourceProperties):
    Activate = attrib(default=None)
    DetectorId = attrib(default=None)
    Format = attrib(default=None)
    Location = attrib(default=None)
    Name = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ThreatIntelSet(Resource):
    """A Threat Intel Set for GuardDuty.

    See Also:
        `AWS Cloud Formation documentation for ThreatIntelSet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-guardduty-threatintelset.html>`_
    """

    RESOURCE_TYPE = "AWS::GuardDuty::ThreatIntelSet"

    Properties: ThreatIntelSetProperties = attrib(
        factory=ThreatIntelSetProperties,
        converter=create_object_converter(ThreatIntelSetProperties),
    )



"""General-use classes to interact with the AmazonMQ service through CloudFormation.

See Also:
    `AWS developer guide for AmazonMQ
    <https://docs.aws.amazon.com/amazon-mq/latest/developer-guide/welcome.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import amazonmq as _raw

# noinspection PyUnresolvedReferences
from .._raw.amazonmq import *



"""Raw representations of every data type in the AWS Inspector service.

See Also:
    `AWS developer guide for Inspector
    <https://docs.aws.amazon.com/inspector/latest/userguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "AssessmentTarget",
    "AssessmentTargetProperties",
    "AssessmentTemplate",
    "AssessmentTemplateProperties",
    "ResourceGroup",
    "ResourceGroupProperties",
]


@attrs(**ATTRSCONFIG)
class AssessmentTargetProperties(ResourceProperties):
    AssessmentTargetName = attrib(default=None)
    ResourceGroupArn = attrib(default=None)


@attrs(**ATTRSCONFIG)
class AssessmentTarget(Resource):
    """A Assessment Target for Inspector.

    See Also:
        `AWS Cloud Formation documentation for AssessmentTarget
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-inspector-assessmenttarget.html>`_
    """

    RESOURCE_TYPE = "AWS::Inspector::AssessmentTarget"

    Properties: AssessmentTargetProperties = attrib(
        factory=AssessmentTargetProperties,
        converter=create_object_converter(AssessmentTargetProperties),
    )


@attrs(**ATTRSCONFIG)
class AssessmentTemplateProperties(ResourceProperties):
    AssessmentTargetArn = attrib(default=None)
    AssessmentTemplateName = attrib(default=None)
    DurationInSeconds = attrib(default=None)
    RulesPackageArns = attrib(default=None)
    UserAttributesForFindings = attrib(default=None)


@attrs(**ATTRSCONFIG)
class AssessmentTemplate(Resource):
    """A Assessment Template for Inspector.

    See Also:
        `AWS Cloud Formation documentation for AssessmentTemplate
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-inspector-assessmenttemplate.html>`_
    """

    RESOURCE_TYPE = "AWS::Inspector::AssessmentTemplate"

    Properties: AssessmentTemplateProperties = attrib(
        factory=AssessmentTemplateProperties,
        converter=create_object_converter(AssessmentTemplateProperties),
    )


@attrs(**ATTRSCONFIG)
class ResourceGroupProperties(ResourceProperties):
    ResourceGroupTags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ResourceGroup(Resource):
    """A Resource Group for Inspector.

    See Also:
        `AWS Cloud Formation documentation for ResourceGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-inspector-resourcegroup.html>`_
    """

    RESOURCE_TYPE = "AWS::Inspector::ResourceGroup"

    Properties: ResourceGroupProperties = attrib(
        factory=ResourceGroupProperties,
        converter=create_object_converter(ResourceGroupProperties),
    )



"""Raw representations of every data type in the AWS WAFRegional service.

See Also:
    `AWS developer guide for WAFRegional
    <https://docs.aws.amazon.com/waf/latest/developerguide/waf-chapter.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "ByteMatchSet",
    "ByteMatchSetProperties",
    "IPSet",
    "IPSetProperties",
    "Rule",
    "RuleProperties",
    "SizeConstraintSet",
    "SizeConstraintSetProperties",
    "SqlInjectionMatchSet",
    "SqlInjectionMatchSetProperties",
    "WebACL",
    "WebACLProperties",
    "WebACLAssociation",
    "WebACLAssociationProperties",
    "XssMatchSet",
    "XssMatchSetProperties",
]


@attrs(**ATTRSCONFIG)
class ByteMatchSetProperties(ResourceProperties):
    ByteMatchTuples = attrib(default=None)
    Name = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ByteMatchSet(Resource):
    """A Byte Match Set for WAFRegional.

    See Also:
        `AWS Cloud Formation documentation for ByteMatchSet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-bytematchset.html>`_
    """

    RESOURCE_TYPE = "AWS::WAFRegional::ByteMatchSet"

    Properties: ByteMatchSetProperties = attrib(
        factory=ByteMatchSetProperties,
        converter=create_object_converter(ByteMatchSetProperties),
    )


@attrs(**ATTRSCONFIG)
class IPSetProperties(ResourceProperties):
    IPSetDescriptors = attrib(default=None)
    Name = attrib(default=None)


@attrs(**ATTRSCONFIG)
class IPSet(Resource):
    """A Ip Set for WAFRegional.

    See Also:
        `AWS Cloud Formation documentation for IPSet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-ipset.html>`_
    """

    RESOURCE_TYPE = "AWS::WAFRegional::IPSet"

    Properties: IPSetProperties = attrib(
        factory=IPSetProperties,
        converter=create_object_converter(IPSetProperties),
    )


@attrs(**ATTRSCONFIG)
class RuleProperties(ResourceProperties):
    MetricName = attrib(default=None)
    Name = attrib(default=None)
    Predicates = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Rule(Resource):
    """A Rule for WAFRegional.

    See Also:
        `AWS Cloud Formation documentation for Rule
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-rule.html>`_
    """

    RESOURCE_TYPE = "AWS::WAFRegional::Rule"

    Properties: RuleProperties = attrib(
        factory=RuleProperties,
        converter=create_object_converter(RuleProperties),
    )


@attrs(**ATTRSCONFIG)
class SizeConstraintSetProperties(ResourceProperties):
    Name = attrib(default=None)
    SizeConstraints = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SizeConstraintSet(Resource):
    """A Size Constraint Set for WAFRegional.

    See Also:
        `AWS Cloud Formation documentation for SizeConstraintSet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-sizeconstraintset.html>`_
    """

    RESOURCE_TYPE = "AWS::WAFRegional::SizeConstraintSet"

    Properties: SizeConstraintSetProperties = attrib(
        factory=SizeConstraintSetProperties,
        converter=create_object_converter(SizeConstraintSetProperties),
    )


@attrs(**ATTRSCONFIG)
class SqlInjectionMatchSetProperties(ResourceProperties):
    Name = attrib(default=None)
    SqlInjectionMatchTuples = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SqlInjectionMatchSet(Resource):
    """A Sql Injection Match Set for WAFRegional.

    See Also:
        `AWS Cloud Formation documentation for SqlInjectionMatchSet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-sqlinjectionmatchset.html>`_
    """

    RESOURCE_TYPE = "AWS::WAFRegional::SqlInjectionMatchSet"

    Properties: SqlInjectionMatchSetProperties = attrib(
        factory=SqlInjectionMatchSetProperties,
        converter=create_object_converter(SqlInjectionMatchSetProperties),
    )


@attrs(**ATTRSCONFIG)
class WebACLProperties(ResourceProperties):
    DefaultAction = attrib(default=None)
    MetricName = attrib(default=None)
    Name = attrib(default=None)
    Rules = attrib(default=None)


@attrs(**ATTRSCONFIG)
class WebACL(Resource):
    """A Web Acl for WAFRegional.

    See Also:
        `AWS Cloud Formation documentation for WebACL
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-webacl.html>`_
    """

    RESOURCE_TYPE = "AWS::WAFRegional::WebACL"

    Properties: WebACLProperties = attrib(
        factory=WebACLProperties,
        converter=create_object_converter(WebACLProperties),
    )


@attrs(**ATTRSCONFIG)
class WebACLAssociationProperties(ResourceProperties):
    ResourceArn = attrib(default=None)
    WebACLId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class WebACLAssociation(Resource):
    """A Web Acl Association for WAFRegional.

    See Also:
        `AWS Cloud Formation documentation for WebACLAssociation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-webaclassociation.html>`_
    """

    RESOURCE_TYPE = "AWS::WAFRegional::WebACLAssociation"

    Properties: WebACLAssociationProperties = attrib(
        factory=WebACLAssociationProperties,
        converter=create_object_converter(WebACLAssociationProperties),
    )


@attrs(**ATTRSCONFIG)
class XssMatchSetProperties(ResourceProperties):
    Name = attrib(default=None)
    XssMatchTuples = attrib(default=None)


@attrs(**ATTRSCONFIG)
class XssMatchSet(Resource):
    """A Xss Match Set for WAFRegional.

    See Also:
        `AWS Cloud Formation documentation for XssMatchSet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-xssmatchset.html>`_
    """

    RESOURCE_TYPE = "AWS::WAFRegional::XssMatchSet"

    Properties: XssMatchSetProperties = attrib(
        factory=XssMatchSetProperties,
        converter=create_object_converter(XssMatchSetProperties),
    )



"""General-use classes to interact with the GameLift service through CloudFormation.

See Also:
    `AWS developer guide for GameLift
    <https://docs.aws.amazon.com/gamelift/latest/developerguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import gamelift as _raw

# noinspection PyUnresolvedReferences
from .._raw.gamelift import *



"""General-use classes to interact with the CloudWatch service through CloudFormation.

See Also:
    `AWS developer guide for CloudWatch
     <http://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/WhatIsCloudWatch.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import cloudwatch as _raw

# noinspection PyUnresolvedReferences
from .._raw.cloudwatch import *


class Alarms:
    """Factory methods to create common types of alarm."""

    # TODO: note: We are providing factory functions, not abstractions. This is because one of our core principles is to use native CFN, so managing an abstraction layer on top of that ourselves is unwise.
    # TODO i don't like shipping in the Java idea of using a class as a namespace. prefer a sub-module, but that conflates with the AWS service namespace we are mimicking?

    @staticmethod
    def high_cpu(threshold):
        """An alarm that triggers when CPU is too high
        (or the underlying metric disappears, indicating the instance is down).
        """
        # TODO I think dimension should be a compulsory attribute
        # TODO share code with the cpu_low_alarm implementation
        return Alarm(
            Properties=dict(
                # TODO sort these in some sensible order, perhaps
                EvaluationPeriods=1,
                Statistic="Average",  # FIXME lookup constant
                Threshold=threshold,
                AlarmDescription="Alarm if CPU too high or metric disappears indicating instance is down",  # TODO tweak
                Period=60,
                Namespace="AWS/EC2",  # FIXME lookup constant?
                ComparisonOperator="GreaterThanThreshold",  # FIXME lookup constant
                MetricName="CPUUtilization"  # TODO Lookup a very long list?
            ),
        )

    @staticmethod
    def low_cpu(threshold):
        """An alarm that triggers when CPU is too low
        (or the underlying metric disappears, indicating the instance is down).
        """
        # TODO I think dimension should be a compulsory attribute
        # TODO share code with the cpu_high_alarm implementation
        return Alarm(
            Properties=dict(
                # TODO sort these in some sensible order, perhaps
                EvaluationPeriods=1,
                Statistic="Average",  # FIXME lookup constant
                Threshold=threshold,
                AlarmDescription="Alarm if CPU too low or metric disappears indicating instance is down",  # TODO tweak
                Period=60,
                Namespace="AWS/EC2",  # FIXME lookup constant?
                ComparisonOperator="LessThanThreshold",  # FIXME lookup constant
                # FIXME check LessThanThreshold is correct, and it;s not LowerThanThreshold or some similar synonym
                MetricName="CPUUtilization"  # TODO Lookup a very long list?
            ),
        )



"""General-use classes to interact with the ServiceCatalog service through CloudFormation.

See Also:
    `AWS developer guide for ServiceCatalog
    <https://docs.aws.amazon.com/servicecatalog/latest/dg/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import servicecatalog as _raw

# noinspection PyUnresolvedReferences
from .._raw.servicecatalog import *



"""Raw representations of every data type in the AWS AutoScaling service.

See Also:
    `AWS developer guide for AutoScaling
    <http://docs.aws.amazon.com/autoscaling/latest/userguide/WhatIsAutoScaling.html>`_

This file is automatically generated, and should not be directly edited.
"""

from typing import Any
from typing import Dict

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "AutoScalingGroup",
    "AutoScalingGroupProperties",
    "LaunchConfiguration",
    "LaunchConfigurationProperties",
    "LifecycleHook",
    "LifecycleHookProperties",
    "ScalingPolicy",
    "ScalingPolicyProperties",
    "ScheduledAction",
    "ScheduledActionProperties",
]


@attrs(**ATTRSCONFIG)
class AutoScalingGroupProperties(ResourceProperties):
    AutoScalingGroupName = attrib(default=None)
    AvailabilityZones = attrib(default=None)
    Cooldown = attrib(default=None)
    DesiredCapacity = attrib(default=None)
    HealthCheckGracePeriod = attrib(default=None)
    HealthCheckType = attrib(default=None)
    InstanceId = attrib(default=None)
    LaunchConfigurationName = attrib(default=None)
    LaunchTemplate = attrib(default=None)
    LifecycleHookSpecificationList = attrib(default=None)
    LoadBalancerNames = attrib(default=None)
    MaxSize = attrib(default=None)
    MetricsCollection = attrib(default=None)
    MinSize = attrib(default=None)
    MixedInstancesPolicy = attrib(default=None)
    NotificationConfigurations = attrib(default=None)
    PlacementGroup = attrib(default=None)
    ServiceLinkedRoleARN = attrib(default=None)
    Tags = attrib(default=None)
    TargetGroupARNs = attrib(default=None)
    TerminationPolicies = attrib(default=None)
    VPCZoneIdentifier = attrib(default=None)


@attrs(**ATTRSCONFIG)
class AutoScalingGroup(Resource):
    """A Auto Scaling Group for AutoScaling.

    See Also:
        `AWS Cloud Formation documentation for AutoScalingGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html>`_
    """

    RESOURCE_TYPE = "AWS::AutoScaling::AutoScalingGroup"

    Properties: AutoScalingGroupProperties = attrib(
        factory=AutoScalingGroupProperties,
        converter=create_object_converter(AutoScalingGroupProperties),
    )

    # NB: CreationPolicy may be set for AutoScalingGroup
    # (unlike most Resource types)
    CreationPolicy: Dict[str, Any] = attrib(factory=dict)

    # NB: UpdatePolicy may be set for AutoScalingGroup
    # (unlike most Resource types)
    UpdatePolicy: Dict[str, Any] = attrib(factory=dict)


@attrs(**ATTRSCONFIG)
class LaunchConfigurationProperties(ResourceProperties):
    AssociatePublicIpAddress = attrib(default=None)
    BlockDeviceMappings = attrib(default=None)
    ClassicLinkVPCId = attrib(default=None)
    ClassicLinkVPCSecurityGroups = attrib(default=None)
    EbsOptimized = attrib(default=None)
    IamInstanceProfile = attrib(default=None)
    ImageId = attrib(default=None)
    InstanceId = attrib(default=None)
    InstanceMonitoring = attrib(default=None)
    InstanceType = attrib(default=None)
    KernelId = attrib(default=None)
    KeyName = attrib(default=None)
    LaunchConfigurationName = attrib(default=None)
    PlacementTenancy = attrib(default=None)
    RamDiskId = attrib(default=None)
    SecurityGroups = attrib(default=None)
    SpotPrice = attrib(default=None)
    UserData = attrib(default=None)


@attrs(**ATTRSCONFIG)
class LaunchConfiguration(Resource):
    """A Launch Configuration for AutoScaling.

    See Also:
        `AWS Cloud Formation documentation for LaunchConfiguration
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html>`_
    """

    RESOURCE_TYPE = "AWS::AutoScaling::LaunchConfiguration"

    Properties: LaunchConfigurationProperties = attrib(
        factory=LaunchConfigurationProperties,
        converter=create_object_converter(LaunchConfigurationProperties),
    )


@attrs(**ATTRSCONFIG)
class LifecycleHookProperties(ResourceProperties):
    AutoScalingGroupName = attrib(default=None)
    DefaultResult = attrib(default=None)
    HeartbeatTimeout = attrib(default=None)
    LifecycleHookName = attrib(default=None)
    LifecycleTransition = attrib(default=None)
    NotificationMetadata = attrib(default=None)
    NotificationTargetARN = attrib(default=None)
    RoleARN = attrib(default=None)


@attrs(**ATTRSCONFIG)
class LifecycleHook(Resource):
    """A Lifecycle Hook for AutoScaling.

    See Also:
        `AWS Cloud Formation documentation for LifecycleHook
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-lifecyclehook.html>`_
    """

    RESOURCE_TYPE = "AWS::AutoScaling::LifecycleHook"

    Properties: LifecycleHookProperties = attrib(
        factory=LifecycleHookProperties,
        converter=create_object_converter(LifecycleHookProperties),
    )


@attrs(**ATTRSCONFIG)
class ScalingPolicyProperties(ResourceProperties):
    AdjustmentType = attrib(default=None)
    AutoScalingGroupName = attrib(default=None)
    Cooldown = attrib(default=None)
    EstimatedInstanceWarmup = attrib(default=None)
    MetricAggregationType = attrib(default=None)
    MinAdjustmentMagnitude = attrib(default=None)
    PolicyType = attrib(default=None)
    ScalingAdjustment = attrib(default=None)
    StepAdjustments = attrib(default=None)
    TargetTrackingConfiguration = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ScalingPolicy(Resource):
    """A Scaling Policy for AutoScaling.

    See Also:
        `AWS Cloud Formation documentation for ScalingPolicy
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-policy.html>`_
    """

    RESOURCE_TYPE = "AWS::AutoScaling::ScalingPolicy"

    Properties: ScalingPolicyProperties = attrib(
        factory=ScalingPolicyProperties,
        converter=create_object_converter(ScalingPolicyProperties),
    )


@attrs(**ATTRSCONFIG)
class ScheduledActionProperties(ResourceProperties):
    AutoScalingGroupName = attrib(default=None)
    DesiredCapacity = attrib(default=None)
    EndTime = attrib(default=None)
    MaxSize = attrib(default=None)
    MinSize = attrib(default=None)
    Recurrence = attrib(default=None)
    StartTime = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ScheduledAction(Resource):
    """A Scheduled Action for AutoScaling.

    See Also:
        `AWS Cloud Formation documentation for ScheduledAction
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-scheduledaction.html>`_
    """

    RESOURCE_TYPE = "AWS::AutoScaling::ScheduledAction"

    Properties: ScheduledActionProperties = attrib(
        factory=ScheduledActionProperties,
        converter=create_object_converter(ScheduledActionProperties),
    )



"""Helper methods for configuring PyYAML."""

# TODO rename this module to avoid confusion with external `yaml` module

from typing import Optional

import yaml
from yaml.resolver import BaseResolver

import flyingcircus


def register_yaml_representers():
    """Configure YAML output from PyYAML."""
    # TODO better to add these to a custom Dumper than pollute the global object?

    # Add marshalling/representers for custom types
    yaml.add_representer(str, represent_string)
    yaml.add_multi_representer(CustomYamlObject, CustomYamlObject.represent_object)

    # Don't silently fail if we try to export something weird
    def unknown_type(dumper, data):
        raise TypeError(
            "{} object cannot be dumped to YAML because it does not "
            "extend CustomYamlObject".format(
                data.__class__.__name__
            )
        )

    yaml.add_multi_representer(object, unknown_type)


class CustomYamlObject(object):
    """Mixin class for objects that have a custom YAML export."""
    __slots__ = []

    def as_yaml_node(self, dumper: yaml.Dumper) -> yaml.Node:
        """Get a representation of this object as a PyYAML node."""
        raise NotImplementedError("as_yaml_node")

    @classmethod
    def represent_object(cls, dumper: yaml.Dumper, data: "CustomYamlObject") -> yaml.Node:
        assert isinstance(data, cls)
        return data.as_yaml_node(dumper)


def represent_string(dumper: yaml.Dumper, data: str, tag: str = BaseResolver.DEFAULT_SCALAR_TAG,
                     basicsep: str = "") -> yaml.ScalarNode:
    """Override the normal string emission rules to produce the most readable text output.

    Args:
        data: The string to dump
        tag: (Optional) The YAML tag to use for this scalar. Defaults to the
            default YAML scalar tag, which does not get printed.
        basicsep: (Optional) The scalar string separator to use for "simple"
            strings (ie. strings where there isn't a more specific rule)
    """
    # TODO test cases

    if '\n' in data:
        # '|' style means literal block style, so line breaks and formatting are retained.
        # This will be especially handy for inline code.
        return dumper.represent_scalar(tag, data, "|")
    elif len(data) > 65:
        # Longer lines will be automatically folded (ie. have line breaks
        # inserted) by PyYAML, which is likely to cause confusion. We
        # compromise by using the literal block style ('|'), which doesn't
        # fold, but does require some special block indicators.
        # TODO need some way to allow folding. Having a helper function probably isn't really good enough. perhaps have the reflow function return a special subclass of string which we can detect here
        return dumper.represent_scalar(tag, data, '|')
    else:
        return dumper.represent_scalar(tag, data, basicsep)


class NonAliasingDumper(yaml.Dumper):
    """A YAML dumper that doesn't use aliases or print anchors.

    The use of aliasing in PyYAML is a little bit arbitrary since it depends
    on object identity, rather than equality. Additionally, the use of node
    aliasing usually makes the output more difficult to understand for a
    human. Finally, AWS CloudFormation doesn't support aliases anyway :-)
    Hence we prefer to not use aliases in our CloudFormation YAML output.

    There is no method to disable aliasing in PyYAML, so we work around this
    by using a subclassed Dumper implementation that clobbers the relevant
    functionality.
    """

    # TODO what about overriding ignore_aliases() instead ?
    # TODO duplicate instances of the same object in a single stack can be confusing, so consider having warning/error functionality if this happens

    def generate_anchor(self, node: yaml.Node) -> Optional[str]:
        # Don't generate anchors at all
        return None

    def serialize_node(self, node: yaml.Node, parent, index):
        # Don't keep track of previously serialised nodes, so any node will appear to be new.
        self.serialized_nodes = {}
        super(NonAliasingDumper, self).serialize_node(node, parent, index)


class AmazonCFNDumper(NonAliasingDumper, yaml.Dumper):
    """A YAML dumper with output customised for AWS CloudFormation."""

    def __init__(self, *args, **kwargs):
        yaml.Dumper.__init__(self, *args, **kwargs)

        self.__cloud_formation_stack: "flyingcircus.core.Stack" = None

    @property
    def cfn_stack(self) -> "flyingcircus.core.Stack":
        """The Cloud Formation stack being exported.

        This should be set by the stack object when it is first represented.
        """
        return self.__cloud_formation_stack

    @cfn_stack.setter
    def cfn_stack(self, value: Optional["flyingcircus.core.Stack"]):
        # Checks
        if value is not None:
            from .core import Stack
            if not isinstance(value, Stack):
                raise TypeError(
                    "The current CloudFormation stack must be a Stack object, "
                    "in order to prevent surprise to users."
                )

            if self.__cloud_formation_stack is not None:
                raise RuntimeError("The current CloudFormation stack is already set!")

        # Set value
        self.__cloud_formation_stack = value

    def choose_scalar_style(self) -> str:
        if self.analysis is None:
            self.analysis = self.analyze_scalar(self.event.value)

        # If an explicit tag is present on a scalar (eg. because it is a Cloud
        # Formation intrinsic function), try to honour a request for plain
        # scalar style. The default behaviour will always clobber this,
        # which is strange.
        if self.event.style == "" and self.event.tag.startswith("!"):
            if not (self.analysis.empty or self.analysis.multiline):
                return ""

        return super().choose_scalar_style()



"""Raw representations of every data type in the AWS Batch service.

See Also:
    `AWS developer guide for Batch
    <https://docs.aws.amazon.com/batch/latest/userguide/what-is-batch.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "ComputeEnvironment",
    "ComputeEnvironmentProperties",
    "JobDefinition",
    "JobDefinitionProperties",
    "JobQueue",
    "JobQueueProperties",
]


@attrs(**ATTRSCONFIG)
class ComputeEnvironmentProperties(ResourceProperties):
    ComputeEnvironmentName = attrib(default=None)
    ComputeResources = attrib(default=None)
    ServiceRole = attrib(default=None)
    State = attrib(default=None)
    Type = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ComputeEnvironment(Resource):
    """A Compute Environment for Batch.

    See Also:
        `AWS Cloud Formation documentation for ComputeEnvironment
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-computeenvironment.html>`_
    """

    RESOURCE_TYPE = "AWS::Batch::ComputeEnvironment"

    Properties: ComputeEnvironmentProperties = attrib(
        factory=ComputeEnvironmentProperties,
        converter=create_object_converter(ComputeEnvironmentProperties),
    )


@attrs(**ATTRSCONFIG)
class JobDefinitionProperties(ResourceProperties):
    ContainerProperties = attrib(default=None)
    JobDefinitionName = attrib(default=None)
    NodeProperties = attrib(default=None)
    Parameters = attrib(default=None)
    RetryStrategy = attrib(default=None)
    Timeout = attrib(default=None)
    Type = attrib(default=None)


@attrs(**ATTRSCONFIG)
class JobDefinition(Resource):
    """A Job Definition for Batch.

    See Also:
        `AWS Cloud Formation documentation for JobDefinition
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobdefinition.html>`_
    """

    RESOURCE_TYPE = "AWS::Batch::JobDefinition"

    Properties: JobDefinitionProperties = attrib(
        factory=JobDefinitionProperties,
        converter=create_object_converter(JobDefinitionProperties),
    )


@attrs(**ATTRSCONFIG)
class JobQueueProperties(ResourceProperties):
    ComputeEnvironmentOrder = attrib(default=None)
    JobQueueName = attrib(default=None)
    Priority = attrib(default=None)
    State = attrib(default=None)


@attrs(**ATTRSCONFIG)
class JobQueue(Resource):
    """A Job Queue for Batch.

    See Also:
        `AWS Cloud Formation documentation for JobQueue
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobqueue.html>`_
    """

    RESOURCE_TYPE = "AWS::Batch::JobQueue"

    Properties: JobQueueProperties = attrib(
        factory=JobQueueProperties,
        converter=create_object_converter(JobQueueProperties),
    )



"""Raw representations of every data type in the AWS IoTAnalytics service.

See Also:
    `AWS developer guide for IoTAnalytics
    <https://docs.aws.amazon.com/appsync/latest/devguide/welcome.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Channel",
    "ChannelProperties",
    "Dataset",
    "DatasetProperties",
    "Datastore",
    "DatastoreProperties",
    "Pipeline",
    "PipelineProperties",
]


@attrs(**ATTRSCONFIG)
class ChannelProperties(ResourceProperties):
    ChannelName = attrib(default=None)
    RetentionPeriod = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Channel(Resource):
    """A Channel for IoTAnalytics.

    See Also:
        `AWS Cloud Formation documentation for Channel
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iotanalytics-channel.html>`_
    """

    RESOURCE_TYPE = "AWS::IoTAnalytics::Channel"

    Properties: ChannelProperties = attrib(
        factory=ChannelProperties,
        converter=create_object_converter(ChannelProperties),
    )


@attrs(**ATTRSCONFIG)
class DatasetProperties(ResourceProperties):
    Actions = attrib(default=None)
    DatasetName = attrib(default=None)
    RetentionPeriod = attrib(default=None)
    Tags = attrib(default=None)
    Triggers = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Dataset(Resource):
    """A Dataset for IoTAnalytics.

    See Also:
        `AWS Cloud Formation documentation for Dataset
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iotanalytics-dataset.html>`_
    """

    RESOURCE_TYPE = "AWS::IoTAnalytics::Dataset"

    Properties: DatasetProperties = attrib(
        factory=DatasetProperties,
        converter=create_object_converter(DatasetProperties),
    )


@attrs(**ATTRSCONFIG)
class DatastoreProperties(ResourceProperties):
    DatastoreName = attrib(default=None)
    RetentionPeriod = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Datastore(Resource):
    """A Datastore for IoTAnalytics.

    See Also:
        `AWS Cloud Formation documentation for Datastore
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iotanalytics-datastore.html>`_
    """

    RESOURCE_TYPE = "AWS::IoTAnalytics::Datastore"

    Properties: DatastoreProperties = attrib(
        factory=DatastoreProperties,
        converter=create_object_converter(DatastoreProperties),
    )


@attrs(**ATTRSCONFIG)
class PipelineProperties(ResourceProperties):
    PipelineActivities = attrib(default=None)
    PipelineName = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Pipeline(Resource):
    """A Pipeline for IoTAnalytics.

    See Also:
        `AWS Cloud Formation documentation for Pipeline
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iotanalytics-pipeline.html>`_
    """

    RESOURCE_TYPE = "AWS::IoTAnalytics::Pipeline"

    Properties: PipelineProperties = attrib(
        factory=PipelineProperties,
        converter=create_object_converter(PipelineProperties),
    )



"""General-use classes to interact with the Batch service through CloudFormation.

See Also:
    `AWS developer guide for Batch
    <https://docs.aws.amazon.com/batch/latest/userguide/what-is-batch.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import batch as _raw

# noinspection PyUnresolvedReferences
from .._raw.batch import *



"""Raw representations of every data type in the AWS GameLift service.

See Also:
    `AWS developer guide for GameLift
    <https://docs.aws.amazon.com/gamelift/latest/developerguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Alias",
    "AliasProperties",
    "Build",
    "BuildProperties",
    "Fleet",
    "FleetProperties",
]


@attrs(**ATTRSCONFIG)
class AliasProperties(ResourceProperties):
    Description = attrib(default=None)
    Name = attrib(default=None)
    RoutingStrategy = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Alias(Resource):
    """A Alias for GameLift.

    See Also:
        `AWS Cloud Formation documentation for Alias
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-alias.html>`_
    """

    RESOURCE_TYPE = "AWS::GameLift::Alias"

    Properties: AliasProperties = attrib(
        factory=AliasProperties,
        converter=create_object_converter(AliasProperties),
    )


@attrs(**ATTRSCONFIG)
class BuildProperties(ResourceProperties):
    Name = attrib(default=None)
    StorageLocation = attrib(default=None)
    Version = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Build(Resource):
    """A Build for GameLift.

    See Also:
        `AWS Cloud Formation documentation for Build
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-build.html>`_
    """

    RESOURCE_TYPE = "AWS::GameLift::Build"

    Properties: BuildProperties = attrib(
        factory=BuildProperties,
        converter=create_object_converter(BuildProperties),
    )


@attrs(**ATTRSCONFIG)
class FleetProperties(ResourceProperties):
    BuildId = attrib(default=None)
    Description = attrib(default=None)
    DesiredEC2Instances = attrib(default=None)
    EC2InboundPermissions = attrib(default=None)
    EC2InstanceType = attrib(default=None)
    LogPaths = attrib(default=None)
    MaxSize = attrib(default=None)
    MinSize = attrib(default=None)
    Name = attrib(default=None)
    ServerLaunchParameters = attrib(default=None)
    ServerLaunchPath = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Fleet(Resource):
    """A Fleet for GameLift.

    See Also:
        `AWS Cloud Formation documentation for Fleet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html>`_
    """

    RESOURCE_TYPE = "AWS::GameLift::Fleet"

    Properties: FleetProperties = attrib(
        factory=FleetProperties,
        converter=create_object_converter(FleetProperties),
    )



"""Raw representations of every data type in the AWS EC2 service.

See Also:
    `AWS developer guide for EC2
    <http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/>`_

This file is automatically generated, and should not be directly edited.
"""

from typing import Any
from typing import Dict

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "CustomerGateway",
    "CustomerGatewayProperties",
    "DHCPOptions",
    "DHCPOptionsProperties",
    "EC2Fleet",
    "EC2FleetProperties",
    "EgressOnlyInternetGateway",
    "EgressOnlyInternetGatewayProperties",
    "EIP",
    "EIPProperties",
    "EIPAssociation",
    "EIPAssociationProperties",
    "FlowLog",
    "FlowLogProperties",
    "Host",
    "HostProperties",
    "Instance",
    "InstanceProperties",
    "InternetGateway",
    "InternetGatewayProperties",
    "LaunchTemplate",
    "LaunchTemplateProperties",
    "NatGateway",
    "NatGatewayProperties",
    "NetworkAcl",
    "NetworkAclProperties",
    "NetworkAclEntry",
    "NetworkAclEntryProperties",
    "NetworkInterface",
    "NetworkInterfaceProperties",
    "NetworkInterfaceAttachment",
    "NetworkInterfaceAttachmentProperties",
    "NetworkInterfacePermission",
    "NetworkInterfacePermissionProperties",
    "PlacementGroup",
    "PlacementGroupProperties",
    "Route",
    "RouteProperties",
    "RouteTable",
    "RouteTableProperties",
    "SecurityGroup",
    "SecurityGroupProperties",
    "SecurityGroupEgress",
    "SecurityGroupEgressProperties",
    "SecurityGroupIngress",
    "SecurityGroupIngressProperties",
    "SpotFleet",
    "SpotFleetProperties",
    "Subnet",
    "SubnetProperties",
    "SubnetCidrBlock",
    "SubnetCidrBlockProperties",
    "SubnetNetworkAclAssociation",
    "SubnetNetworkAclAssociationProperties",
    "SubnetRouteTableAssociation",
    "SubnetRouteTableAssociationProperties",
    "TransitGateway",
    "TransitGatewayProperties",
    "TransitGatewayAttachment",
    "TransitGatewayAttachmentProperties",
    "TransitGatewayRoute",
    "TransitGatewayRouteProperties",
    "TransitGatewayRouteTable",
    "TransitGatewayRouteTableProperties",
    "TransitGatewayRouteTableAssociation",
    "TransitGatewayRouteTableAssociationProperties",
    "TransitGatewayRouteTablePropagation",
    "TransitGatewayRouteTablePropagationProperties",
    "TrunkInterfaceAssociation",
    "TrunkInterfaceAssociationProperties",
    "Volume",
    "VolumeProperties",
    "VolumeAttachment",
    "VolumeAttachmentProperties",
    "VPC",
    "VPCProperties",
    "VPCCidrBlock",
    "VPCCidrBlockProperties",
    "VPCDHCPOptionsAssociation",
    "VPCDHCPOptionsAssociationProperties",
    "VPCEndpoint",
    "VPCEndpointProperties",
    "VPCEndpointConnectionNotification",
    "VPCEndpointConnectionNotificationProperties",
    "VPCEndpointServicePermissions",
    "VPCEndpointServicePermissionsProperties",
    "VPCGatewayAttachment",
    "VPCGatewayAttachmentProperties",
    "VPCPeeringConnection",
    "VPCPeeringConnectionProperties",
    "VPNConnection",
    "VPNConnectionProperties",
    "VPNConnectionRoute",
    "VPNConnectionRouteProperties",
    "VPNGateway",
    "VPNGatewayProperties",
    "VPNGatewayRoutePropagation",
    "VPNGatewayRoutePropagationProperties",
]


@attrs(**ATTRSCONFIG)
class CustomerGatewayProperties(ResourceProperties):
    BgpAsn = attrib(default=None)
    IpAddress = attrib(default=None)
    Tags = attrib(default=None)
    Type = attrib(default=None)


@attrs(**ATTRSCONFIG)
class CustomerGateway(Resource):
    """A Customer Gateway for EC2.

    See Also:
        `AWS Cloud Formation documentation for CustomerGateway
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-customer-gateway.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::CustomerGateway"

    Properties: CustomerGatewayProperties = attrib(
        factory=CustomerGatewayProperties,
        converter=create_object_converter(CustomerGatewayProperties),
    )


@attrs(**ATTRSCONFIG)
class DHCPOptionsProperties(ResourceProperties):
    DomainName = attrib(default=None)
    DomainNameServers = attrib(default=None)
    NetbiosNameServers = attrib(default=None)
    NetbiosNodeType = attrib(default=None)
    NtpServers = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DHCPOptions(Resource):
    """A Dhcp Options for EC2.

    See Also:
        `AWS Cloud Formation documentation for DHCPOptions
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-dhcp-options.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::DHCPOptions"

    Properties: DHCPOptionsProperties = attrib(
        factory=DHCPOptionsProperties,
        converter=create_object_converter(DHCPOptionsProperties),
    )


@attrs(**ATTRSCONFIG)
class EC2FleetProperties(ResourceProperties):
    ExcessCapacityTerminationPolicy = attrib(default=None)
    LaunchTemplateConfigs = attrib(default=None)
    OnDemandOptions = attrib(default=None)
    ReplaceUnhealthyInstances = attrib(default=None)
    SpotOptions = attrib(default=None)
    TagSpecifications = attrib(default=None)
    TargetCapacitySpecification = attrib(default=None)
    TerminateInstancesWithExpiration = attrib(default=None)
    Type = attrib(default=None)
    ValidFrom = attrib(default=None)
    ValidUntil = attrib(default=None)


@attrs(**ATTRSCONFIG)
class EC2Fleet(Resource):
    """A Ec2 Fleet for EC2.

    See Also:
        `AWS Cloud Formation documentation for EC2Fleet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-ec2fleet.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::EC2Fleet"

    Properties: EC2FleetProperties = attrib(
        factory=EC2FleetProperties,
        converter=create_object_converter(EC2FleetProperties),
    )


@attrs(**ATTRSCONFIG)
class EIPProperties(ResourceProperties):
    Domain = attrib(default=None)
    InstanceId = attrib(default=None)
    PublicIpv4Pool = attrib(default=None)


@attrs(**ATTRSCONFIG)
class EIP(Resource):
    """A Eip for EC2.

    See Also:
        `AWS Cloud Formation documentation for EIP
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-eip.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::EIP"

    Properties: EIPProperties = attrib(
        factory=EIPProperties,
        converter=create_object_converter(EIPProperties),
    )


@attrs(**ATTRSCONFIG)
class EIPAssociationProperties(ResourceProperties):
    AllocationId = attrib(default=None)
    EIP = attrib(default=None)
    InstanceId = attrib(default=None)
    NetworkInterfaceId = attrib(default=None)
    PrivateIpAddress = attrib(default=None)


@attrs(**ATTRSCONFIG)
class EIPAssociation(Resource):
    """A Eip Association for EC2.

    See Also:
        `AWS Cloud Formation documentation for EIPAssociation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-eip-association.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::EIPAssociation"

    Properties: EIPAssociationProperties = attrib(
        factory=EIPAssociationProperties,
        converter=create_object_converter(EIPAssociationProperties),
    )


@attrs(**ATTRSCONFIG)
class EgressOnlyInternetGatewayProperties(ResourceProperties):
    VpcId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class EgressOnlyInternetGateway(Resource):
    """A Egress Only Internet Gateway for EC2.

    See Also:
        `AWS Cloud Formation documentation for EgressOnlyInternetGateway
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-egressonlyinternetgateway.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::EgressOnlyInternetGateway"

    Properties: EgressOnlyInternetGatewayProperties = attrib(
        factory=EgressOnlyInternetGatewayProperties,
        converter=create_object_converter(EgressOnlyInternetGatewayProperties),
    )


@attrs(**ATTRSCONFIG)
class FlowLogProperties(ResourceProperties):
    DeliverLogsPermissionArn = attrib(default=None)
    LogDestination = attrib(default=None)
    LogDestinationType = attrib(default=None)
    LogGroupName = attrib(default=None)
    ResourceId = attrib(default=None)
    ResourceType = attrib(default=None)
    TrafficType = attrib(default=None)


@attrs(**ATTRSCONFIG)
class FlowLog(Resource):
    """A Flow Log for EC2.

    See Also:
        `AWS Cloud Formation documentation for FlowLog
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-flowlog.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::FlowLog"

    Properties: FlowLogProperties = attrib(
        factory=FlowLogProperties,
        converter=create_object_converter(FlowLogProperties),
    )


@attrs(**ATTRSCONFIG)
class HostProperties(ResourceProperties):
    AutoPlacement = attrib(default=None)
    AvailabilityZone = attrib(default=None)
    InstanceType = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Host(Resource):
    """A Host for EC2.

    See Also:
        `AWS Cloud Formation documentation for Host
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-host.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::Host"

    Properties: HostProperties = attrib(
        factory=HostProperties,
        converter=create_object_converter(HostProperties),
    )


@attrs(**ATTRSCONFIG)
class InstanceProperties(ResourceProperties):
    AdditionalInfo = attrib(default=None)
    Affinity = attrib(default=None)
    AvailabilityZone = attrib(default=None)
    BlockDeviceMappings = attrib(default=None)
    CreditSpecification = attrib(default=None)
    DisableApiTermination = attrib(default=None)
    EbsOptimized = attrib(default=None)
    ElasticGpuSpecifications = attrib(default=None)
    ElasticInferenceAccelerators = attrib(default=None)
    HostId = attrib(default=None)
    IamInstanceProfile = attrib(default=None)
    ImageId = attrib(default=None)
    InstanceInitiatedShutdownBehavior = attrib(default=None)
    InstanceType = attrib(default=None)
    Ipv6AddressCount = attrib(default=None)
    Ipv6Addresses = attrib(default=None)
    KernelId = attrib(default=None)
    KeyName = attrib(default=None)
    LaunchTemplate = attrib(default=None)
    LicenseSpecifications = attrib(default=None)
    Monitoring = attrib(default=None)
    NetworkInterfaces = attrib(default=None)
    PlacementGroupName = attrib(default=None)
    PrivateIpAddress = attrib(default=None)
    RamdiskId = attrib(default=None)
    SecurityGroupIds = attrib(default=None)
    SecurityGroups = attrib(default=None)
    SourceDestCheck = attrib(default=None)
    SsmAssociations = attrib(default=None)
    SubnetId = attrib(default=None)
    Tags = attrib(default=None)
    Tenancy = attrib(default=None)
    UserData = attrib(default=None)
    Volumes = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Instance(Resource):
    """A Instance for EC2.

    See Also:
        `AWS Cloud Formation documentation for Instance
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::Instance"

    Properties: InstanceProperties = attrib(
        factory=InstanceProperties,
        converter=create_object_converter(InstanceProperties),
    )

    # NB: CreationPolicy may be set for Instance
    # (unlike most Resource types)
    CreationPolicy: Dict[str, Any] = attrib(factory=dict)


@attrs(**ATTRSCONFIG)
class InternetGatewayProperties(ResourceProperties):
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class InternetGateway(Resource):
    """A Internet Gateway for EC2.

    See Also:
        `AWS Cloud Formation documentation for InternetGateway
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-internetgateway.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::InternetGateway"

    Properties: InternetGatewayProperties = attrib(
        factory=InternetGatewayProperties,
        converter=create_object_converter(InternetGatewayProperties),
    )


@attrs(**ATTRSCONFIG)
class LaunchTemplateProperties(ResourceProperties):
    LaunchTemplateData = attrib(default=None)
    LaunchTemplateName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class LaunchTemplate(Resource):
    """A Launch Template for EC2.

    See Also:
        `AWS Cloud Formation documentation for LaunchTemplate
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-launchtemplate.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::LaunchTemplate"

    Properties: LaunchTemplateProperties = attrib(
        factory=LaunchTemplateProperties,
        converter=create_object_converter(LaunchTemplateProperties),
    )


@attrs(**ATTRSCONFIG)
class NatGatewayProperties(ResourceProperties):
    AllocationId = attrib(default=None)
    SubnetId = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class NatGateway(Resource):
    """A Nat Gateway for EC2.

    See Also:
        `AWS Cloud Formation documentation for NatGateway
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-natgateway.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::NatGateway"

    Properties: NatGatewayProperties = attrib(
        factory=NatGatewayProperties,
        converter=create_object_converter(NatGatewayProperties),
    )


@attrs(**ATTRSCONFIG)
class NetworkAclProperties(ResourceProperties):
    Tags = attrib(default=None)
    VpcId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class NetworkAcl(Resource):
    """A Network Acl for EC2.

    See Also:
        `AWS Cloud Formation documentation for NetworkAcl
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::NetworkAcl"

    Properties: NetworkAclProperties = attrib(
        factory=NetworkAclProperties,
        converter=create_object_converter(NetworkAclProperties),
    )


@attrs(**ATTRSCONFIG)
class NetworkAclEntryProperties(ResourceProperties):
    CidrBlock = attrib(default=None)
    Egress = attrib(default=None)
    Icmp = attrib(default=None)
    Ipv6CidrBlock = attrib(default=None)
    NetworkAclId = attrib(default=None)
    PortRange = attrib(default=None)
    Protocol = attrib(default=None)
    RuleAction = attrib(default=None)
    RuleNumber = attrib(default=None)


@attrs(**ATTRSCONFIG)
class NetworkAclEntry(Resource):
    """A Network Acl Entry for EC2.

    See Also:
        `AWS Cloud Formation documentation for NetworkAclEntry
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl-entry.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::NetworkAclEntry"

    Properties: NetworkAclEntryProperties = attrib(
        factory=NetworkAclEntryProperties,
        converter=create_object_converter(NetworkAclEntryProperties),
    )


@attrs(**ATTRSCONFIG)
class NetworkInterfaceProperties(ResourceProperties):
    Description = attrib(default=None)
    GroupSet = attrib(default=None)
    InterfaceType = attrib(default=None)
    Ipv6AddressCount = attrib(default=None)
    Ipv6Addresses = attrib(default=None)
    PrivateIpAddress = attrib(default=None)
    PrivateIpAddresses = attrib(default=None)
    SecondaryPrivateIpAddressCount = attrib(default=None)
    SourceDestCheck = attrib(default=None)
    SubnetId = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class NetworkInterface(Resource):
    """A Network Interface for EC2.

    See Also:
        `AWS Cloud Formation documentation for NetworkInterface
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::NetworkInterface"

    Properties: NetworkInterfaceProperties = attrib(
        factory=NetworkInterfaceProperties,
        converter=create_object_converter(NetworkInterfaceProperties),
    )


@attrs(**ATTRSCONFIG)
class NetworkInterfaceAttachmentProperties(ResourceProperties):
    DeleteOnTermination = attrib(default=None)
    DeviceIndex = attrib(default=None)
    InstanceId = attrib(default=None)
    NetworkInterfaceId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class NetworkInterfaceAttachment(Resource):
    """A Network Interface Attachment for EC2.

    See Also:
        `AWS Cloud Formation documentation for NetworkInterfaceAttachment
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface-attachment.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::NetworkInterfaceAttachment"

    Properties: NetworkInterfaceAttachmentProperties = attrib(
        factory=NetworkInterfaceAttachmentProperties,
        converter=create_object_converter(NetworkInterfaceAttachmentProperties),
    )


@attrs(**ATTRSCONFIG)
class NetworkInterfacePermissionProperties(ResourceProperties):
    AwsAccountId = attrib(default=None)
    NetworkInterfaceId = attrib(default=None)
    Permission = attrib(default=None)


@attrs(**ATTRSCONFIG)
class NetworkInterfacePermission(Resource):
    """A Network Interface Permission for EC2.

    See Also:
        `AWS Cloud Formation documentation for NetworkInterfacePermission
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-networkinterfacepermission.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::NetworkInterfacePermission"

    Properties: NetworkInterfacePermissionProperties = attrib(
        factory=NetworkInterfacePermissionProperties,
        converter=create_object_converter(NetworkInterfacePermissionProperties),
    )


@attrs(**ATTRSCONFIG)
class PlacementGroupProperties(ResourceProperties):
    Strategy = attrib(default=None)


@attrs(**ATTRSCONFIG)
class PlacementGroup(Resource):
    """A Placement Group for EC2.

    See Also:
        `AWS Cloud Formation documentation for PlacementGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-placementgroup.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::PlacementGroup"

    Properties: PlacementGroupProperties = attrib(
        factory=PlacementGroupProperties,
        converter=create_object_converter(PlacementGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class RouteProperties(ResourceProperties):
    DestinationCidrBlock = attrib(default=None)
    DestinationIpv6CidrBlock = attrib(default=None)
    EgressOnlyInternetGatewayId = attrib(default=None)
    GatewayId = attrib(default=None)
    InstanceId = attrib(default=None)
    NatGatewayId = attrib(default=None)
    NetworkInterfaceId = attrib(default=None)
    RouteTableId = attrib(default=None)
    VpcPeeringConnectionId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Route(Resource):
    """A Route for EC2.

    See Also:
        `AWS Cloud Formation documentation for Route
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::Route"

    Properties: RouteProperties = attrib(
        factory=RouteProperties,
        converter=create_object_converter(RouteProperties),
    )


@attrs(**ATTRSCONFIG)
class RouteTableProperties(ResourceProperties):
    Tags = attrib(default=None)
    VpcId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class RouteTable(Resource):
    """A Route Table for EC2.

    See Also:
        `AWS Cloud Formation documentation for RouteTable
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route-table.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::RouteTable"

    Properties: RouteTableProperties = attrib(
        factory=RouteTableProperties,
        converter=create_object_converter(RouteTableProperties),
    )


@attrs(**ATTRSCONFIG)
class SecurityGroupProperties(ResourceProperties):
    GroupDescription = attrib(default=None)
    GroupName = attrib(default=None)
    SecurityGroupEgress = attrib(default=None)
    SecurityGroupIngress = attrib(default=None)
    Tags = attrib(default=None)
    VpcId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SecurityGroup(Resource):
    """A Security Group for EC2.

    See Also:
        `AWS Cloud Formation documentation for SecurityGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::SecurityGroup"

    Properties: SecurityGroupProperties = attrib(
        factory=SecurityGroupProperties,
        converter=create_object_converter(SecurityGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class SecurityGroupEgressProperties(ResourceProperties):
    CidrIp = attrib(default=None)
    CidrIpv6 = attrib(default=None)
    Description = attrib(default=None)
    DestinationPrefixListId = attrib(default=None)
    DestinationSecurityGroupId = attrib(default=None)
    FromPort = attrib(default=None)
    GroupId = attrib(default=None)
    IpProtocol = attrib(default=None)
    ToPort = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SecurityGroupEgress(Resource):
    """A Security Group Egress for EC2.

    See Also:
        `AWS Cloud Formation documentation for SecurityGroupEgress
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-security-group-egress.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::SecurityGroupEgress"

    Properties: SecurityGroupEgressProperties = attrib(
        factory=SecurityGroupEgressProperties,
        converter=create_object_converter(SecurityGroupEgressProperties),
    )


@attrs(**ATTRSCONFIG)
class SecurityGroupIngressProperties(ResourceProperties):
    CidrIp = attrib(default=None)
    CidrIpv6 = attrib(default=None)
    Description = attrib(default=None)
    FromPort = attrib(default=None)
    GroupId = attrib(default=None)
    GroupName = attrib(default=None)
    IpProtocol = attrib(default=None)
    SourcePrefixListId = attrib(default=None)
    SourceSecurityGroupId = attrib(default=None)
    SourceSecurityGroupName = attrib(default=None)
    SourceSecurityGroupOwnerId = attrib(default=None)
    ToPort = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SecurityGroupIngress(Resource):
    """A Security Group Ingress for EC2.

    See Also:
        `AWS Cloud Formation documentation for SecurityGroupIngress
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::SecurityGroupIngress"

    Properties: SecurityGroupIngressProperties = attrib(
        factory=SecurityGroupIngressProperties,
        converter=create_object_converter(SecurityGroupIngressProperties),
    )


@attrs(**ATTRSCONFIG)
class SpotFleetProperties(ResourceProperties):
    SpotFleetRequestConfigData = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SpotFleet(Resource):
    """A Spot Fleet for EC2.

    See Also:
        `AWS Cloud Formation documentation for SpotFleet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-spotfleet.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::SpotFleet"

    Properties: SpotFleetProperties = attrib(
        factory=SpotFleetProperties,
        converter=create_object_converter(SpotFleetProperties),
    )


@attrs(**ATTRSCONFIG)
class SubnetProperties(ResourceProperties):
    AssignIpv6AddressOnCreation = attrib(default=None)
    AvailabilityZone = attrib(default=None)
    CidrBlock = attrib(default=None)
    Ipv6CidrBlock = attrib(default=None)
    MapPublicIpOnLaunch = attrib(default=None)
    Tags = attrib(default=None)
    VpcId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Subnet(Resource):
    """A Subnet for EC2.

    See Also:
        `AWS Cloud Formation documentation for Subnet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::Subnet"

    Properties: SubnetProperties = attrib(
        factory=SubnetProperties,
        converter=create_object_converter(SubnetProperties),
    )


@attrs(**ATTRSCONFIG)
class SubnetCidrBlockProperties(ResourceProperties):
    Ipv6CidrBlock = attrib(default=None)
    SubnetId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SubnetCidrBlock(Resource):
    """A Subnet Cidr Block for EC2.

    See Also:
        `AWS Cloud Formation documentation for SubnetCidrBlock
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnetcidrblock.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::SubnetCidrBlock"

    Properties: SubnetCidrBlockProperties = attrib(
        factory=SubnetCidrBlockProperties,
        converter=create_object_converter(SubnetCidrBlockProperties),
    )


@attrs(**ATTRSCONFIG)
class SubnetNetworkAclAssociationProperties(ResourceProperties):
    NetworkAclId = attrib(default=None)
    SubnetId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SubnetNetworkAclAssociation(Resource):
    """A Subnet Network Acl Association for EC2.

    See Also:
        `AWS Cloud Formation documentation for SubnetNetworkAclAssociation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet-network-acl-assoc.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::SubnetNetworkAclAssociation"

    Properties: SubnetNetworkAclAssociationProperties = attrib(
        factory=SubnetNetworkAclAssociationProperties,
        converter=create_object_converter(SubnetNetworkAclAssociationProperties),
    )


@attrs(**ATTRSCONFIG)
class SubnetRouteTableAssociationProperties(ResourceProperties):
    RouteTableId = attrib(default=None)
    SubnetId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SubnetRouteTableAssociation(Resource):
    """A Subnet Route Table Association for EC2.

    See Also:
        `AWS Cloud Formation documentation for SubnetRouteTableAssociation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet-route-table-assoc.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::SubnetRouteTableAssociation"

    Properties: SubnetRouteTableAssociationProperties = attrib(
        factory=SubnetRouteTableAssociationProperties,
        converter=create_object_converter(SubnetRouteTableAssociationProperties),
    )


@attrs(**ATTRSCONFIG)
class TransitGatewayProperties(ResourceProperties):
    AmazonSideAsn = attrib(default=None)
    AutoAcceptSharedAttachments = attrib(default=None)
    DefaultRouteTableAssociation = attrib(default=None)
    DefaultRouteTablePropagation = attrib(default=None)
    Description = attrib(default=None)
    DnsSupport = attrib(default=None)
    Tags = attrib(default=None)
    VpnEcmpSupport = attrib(default=None)


@attrs(**ATTRSCONFIG)
class TransitGateway(Resource):
    """A Transit Gateway for EC2.

    See Also:
        `AWS Cloud Formation documentation for TransitGateway
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-transitgateway.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::TransitGateway"

    Properties: TransitGatewayProperties = attrib(
        factory=TransitGatewayProperties,
        converter=create_object_converter(TransitGatewayProperties),
    )


@attrs(**ATTRSCONFIG)
class TransitGatewayAttachmentProperties(ResourceProperties):
    SubnetIds = attrib(default=None)
    Tags = attrib(default=None)
    TransitGatewayId = attrib(default=None)
    VpcId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class TransitGatewayAttachment(Resource):
    """A Transit Gateway Attachment for EC2.

    See Also:
        `AWS Cloud Formation documentation for TransitGatewayAttachment
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-transitgatewayattachment.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::TransitGatewayAttachment"

    Properties: TransitGatewayAttachmentProperties = attrib(
        factory=TransitGatewayAttachmentProperties,
        converter=create_object_converter(TransitGatewayAttachmentProperties),
    )


@attrs(**ATTRSCONFIG)
class TransitGatewayRouteProperties(ResourceProperties):
    Blackhole = attrib(default=None)
    DestinationCidrBlock = attrib(default=None)
    TransitGatewayAttachmentId = attrib(default=None)
    TransitGatewayRouteTableId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class TransitGatewayRoute(Resource):
    """A Transit Gateway Route for EC2.

    See Also:
        `AWS Cloud Formation documentation for TransitGatewayRoute
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-transitgatewayroute.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::TransitGatewayRoute"

    Properties: TransitGatewayRouteProperties = attrib(
        factory=TransitGatewayRouteProperties,
        converter=create_object_converter(TransitGatewayRouteProperties),
    )


@attrs(**ATTRSCONFIG)
class TransitGatewayRouteTableProperties(ResourceProperties):
    Tags = attrib(default=None)
    TransitGatewayId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class TransitGatewayRouteTable(Resource):
    """A Transit Gateway Route Table for EC2.

    See Also:
        `AWS Cloud Formation documentation for TransitGatewayRouteTable
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-transitgatewayroutetable.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::TransitGatewayRouteTable"

    Properties: TransitGatewayRouteTableProperties = attrib(
        factory=TransitGatewayRouteTableProperties,
        converter=create_object_converter(TransitGatewayRouteTableProperties),
    )


@attrs(**ATTRSCONFIG)
class TransitGatewayRouteTableAssociationProperties(ResourceProperties):
    TransitGatewayAttachmentId = attrib(default=None)
    TransitGatewayRouteTableId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class TransitGatewayRouteTableAssociation(Resource):
    """A Transit Gateway Route Table Association for EC2.

    See Also:
        `AWS Cloud Formation documentation for TransitGatewayRouteTableAssociation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-transitgatewayroutetableassociation.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::TransitGatewayRouteTableAssociation"

    Properties: TransitGatewayRouteTableAssociationProperties = attrib(
        factory=TransitGatewayRouteTableAssociationProperties,
        converter=create_object_converter(TransitGatewayRouteTableAssociationProperties),
    )


@attrs(**ATTRSCONFIG)
class TransitGatewayRouteTablePropagationProperties(ResourceProperties):
    TransitGatewayAttachmentId = attrib(default=None)
    TransitGatewayRouteTableId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class TransitGatewayRouteTablePropagation(Resource):
    """A Transit Gateway Route Table Propagation for EC2.

    See Also:
        `AWS Cloud Formation documentation for TransitGatewayRouteTablePropagation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-transitgatewayroutetablepropagation.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::TransitGatewayRouteTablePropagation"

    Properties: TransitGatewayRouteTablePropagationProperties = attrib(
        factory=TransitGatewayRouteTablePropagationProperties,
        converter=create_object_converter(TransitGatewayRouteTablePropagationProperties),
    )


@attrs(**ATTRSCONFIG)
class TrunkInterfaceAssociationProperties(ResourceProperties):
    BranchInterfaceId = attrib(default=None)
    GREKey = attrib(default=None)
    TrunkInterfaceId = attrib(default=None)
    VLANId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class TrunkInterfaceAssociation(Resource):
    """A Trunk Interface Association for EC2.

    See Also:
        `AWS Cloud Formation documentation for TrunkInterfaceAssociation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-trunkinterfaceassociation.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::TrunkInterfaceAssociation"

    Properties: TrunkInterfaceAssociationProperties = attrib(
        factory=TrunkInterfaceAssociationProperties,
        converter=create_object_converter(TrunkInterfaceAssociationProperties),
    )


@attrs(**ATTRSCONFIG)
class VPCProperties(ResourceProperties):
    CidrBlock = attrib(default=None)
    EnableDnsHostnames = attrib(default=None)
    EnableDnsSupport = attrib(default=None)
    InstanceTenancy = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class VPC(Resource):
    """A Vpc for EC2.

    See Also:
        `AWS Cloud Formation documentation for VPC
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::VPC"

    Properties: VPCProperties = attrib(
        factory=VPCProperties,
        converter=create_object_converter(VPCProperties),
    )


@attrs(**ATTRSCONFIG)
class VPCCidrBlockProperties(ResourceProperties):
    AmazonProvidedIpv6CidrBlock = attrib(default=None)
    CidrBlock = attrib(default=None)
    VpcId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class VPCCidrBlock(Resource):
    """A Vpc Cidr Block for EC2.

    See Also:
        `AWS Cloud Formation documentation for VPCCidrBlock
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpccidrblock.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::VPCCidrBlock"

    Properties: VPCCidrBlockProperties = attrib(
        factory=VPCCidrBlockProperties,
        converter=create_object_converter(VPCCidrBlockProperties),
    )


@attrs(**ATTRSCONFIG)
class VPCDHCPOptionsAssociationProperties(ResourceProperties):
    DhcpOptionsId = attrib(default=None)
    VpcId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class VPCDHCPOptionsAssociation(Resource):
    """A Vpcdhcp Options Association for EC2.

    See Also:
        `AWS Cloud Formation documentation for VPCDHCPOptionsAssociation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc-dhcp-options-assoc.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::VPCDHCPOptionsAssociation"

    Properties: VPCDHCPOptionsAssociationProperties = attrib(
        factory=VPCDHCPOptionsAssociationProperties,
        converter=create_object_converter(VPCDHCPOptionsAssociationProperties),
    )


@attrs(**ATTRSCONFIG)
class VPCEndpointProperties(ResourceProperties):
    PolicyDocument = attrib(default=None)
    PrivateDnsEnabled = attrib(default=None)
    RouteTableIds = attrib(default=None)
    SecurityGroupIds = attrib(default=None)
    ServiceName = attrib(default=None)
    SubnetIds = attrib(default=None)
    VpcEndpointType = attrib(default=None)
    VpcId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class VPCEndpoint(Resource):
    """A Vpc Endpoint for EC2.

    See Also:
        `AWS Cloud Formation documentation for VPCEndpoint
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcendpoint.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::VPCEndpoint"

    Properties: VPCEndpointProperties = attrib(
        factory=VPCEndpointProperties,
        converter=create_object_converter(VPCEndpointProperties),
    )


@attrs(**ATTRSCONFIG)
class VPCEndpointConnectionNotificationProperties(ResourceProperties):
    ConnectionEvents = attrib(default=None)
    ConnectionNotificationArn = attrib(default=None)
    ServiceId = attrib(default=None)
    VPCEndpointId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class VPCEndpointConnectionNotification(Resource):
    """A Vpc Endpoint Connection Notification for EC2.

    See Also:
        `AWS Cloud Formation documentation for VPCEndpointConnectionNotification
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcendpointconnectionnotification.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::VPCEndpointConnectionNotification"

    Properties: VPCEndpointConnectionNotificationProperties = attrib(
        factory=VPCEndpointConnectionNotificationProperties,
        converter=create_object_converter(VPCEndpointConnectionNotificationProperties),
    )


@attrs(**ATTRSCONFIG)
class VPCEndpointServicePermissionsProperties(ResourceProperties):
    AllowedPrincipals = attrib(default=None)
    ServiceId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class VPCEndpointServicePermissions(Resource):
    """A Vpc Endpoint Service Permissions for EC2.

    See Also:
        `AWS Cloud Formation documentation for VPCEndpointServicePermissions
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcendpointservicepermissions.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::VPCEndpointServicePermissions"

    Properties: VPCEndpointServicePermissionsProperties = attrib(
        factory=VPCEndpointServicePermissionsProperties,
        converter=create_object_converter(VPCEndpointServicePermissionsProperties),
    )


@attrs(**ATTRSCONFIG)
class VPCGatewayAttachmentProperties(ResourceProperties):
    InternetGatewayId = attrib(default=None)
    VpcId = attrib(default=None)
    VpnGatewayId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class VPCGatewayAttachment(Resource):
    """A Vpc Gateway Attachment for EC2.

    See Also:
        `AWS Cloud Formation documentation for VPCGatewayAttachment
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc-gateway-attachment.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::VPCGatewayAttachment"

    Properties: VPCGatewayAttachmentProperties = attrib(
        factory=VPCGatewayAttachmentProperties,
        converter=create_object_converter(VPCGatewayAttachmentProperties),
    )


@attrs(**ATTRSCONFIG)
class VPCPeeringConnectionProperties(ResourceProperties):
    PeerOwnerId = attrib(default=None)
    PeerRegion = attrib(default=None)
    PeerRoleArn = attrib(default=None)
    PeerVpcId = attrib(default=None)
    Tags = attrib(default=None)
    VpcId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class VPCPeeringConnection(Resource):
    """A Vpc Peering Connection for EC2.

    See Also:
        `AWS Cloud Formation documentation for VPCPeeringConnection
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcpeeringconnection.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::VPCPeeringConnection"

    Properties: VPCPeeringConnectionProperties = attrib(
        factory=VPCPeeringConnectionProperties,
        converter=create_object_converter(VPCPeeringConnectionProperties),
    )


@attrs(**ATTRSCONFIG)
class VPNConnectionProperties(ResourceProperties):
    CustomerGatewayId = attrib(default=None)
    StaticRoutesOnly = attrib(default=None)
    Tags = attrib(default=None)
    Type = attrib(default=None)
    VpnGatewayId = attrib(default=None)
    VpnTunnelOptionsSpecifications = attrib(default=None)


@attrs(**ATTRSCONFIG)
class VPNConnection(Resource):
    """A Vpn Connection for EC2.

    See Also:
        `AWS Cloud Formation documentation for VPNConnection
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-connection.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::VPNConnection"

    Properties: VPNConnectionProperties = attrib(
        factory=VPNConnectionProperties,
        converter=create_object_converter(VPNConnectionProperties),
    )


@attrs(**ATTRSCONFIG)
class VPNConnectionRouteProperties(ResourceProperties):
    DestinationCidrBlock = attrib(default=None)
    VpnConnectionId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class VPNConnectionRoute(Resource):
    """A Vpn Connection Route for EC2.

    See Also:
        `AWS Cloud Formation documentation for VPNConnectionRoute
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-connection-route.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::VPNConnectionRoute"

    Properties: VPNConnectionRouteProperties = attrib(
        factory=VPNConnectionRouteProperties,
        converter=create_object_converter(VPNConnectionRouteProperties),
    )


@attrs(**ATTRSCONFIG)
class VPNGatewayProperties(ResourceProperties):
    AmazonSideAsn = attrib(default=None)
    Tags = attrib(default=None)
    Type = attrib(default=None)


@attrs(**ATTRSCONFIG)
class VPNGateway(Resource):
    """A Vpn Gateway for EC2.

    See Also:
        `AWS Cloud Formation documentation for VPNGateway
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-gateway.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::VPNGateway"

    Properties: VPNGatewayProperties = attrib(
        factory=VPNGatewayProperties,
        converter=create_object_converter(VPNGatewayProperties),
    )


@attrs(**ATTRSCONFIG)
class VPNGatewayRoutePropagationProperties(ResourceProperties):
    RouteTableIds = attrib(default=None)
    VpnGatewayId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class VPNGatewayRoutePropagation(Resource):
    """A Vpn Gateway Route Propagation for EC2.

    See Also:
        `AWS Cloud Formation documentation for VPNGatewayRoutePropagation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-gatewayrouteprop.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::VPNGatewayRoutePropagation"

    Properties: VPNGatewayRoutePropagationProperties = attrib(
        factory=VPNGatewayRoutePropagationProperties,
        converter=create_object_converter(VPNGatewayRoutePropagationProperties),
    )


@attrs(**ATTRSCONFIG)
class VolumeProperties(ResourceProperties):
    AutoEnableIO = attrib(default=None)
    AvailabilityZone = attrib(default=None)
    Encrypted = attrib(default=None)
    Iops = attrib(default=None)
    KmsKeyId = attrib(default=None)
    Size = attrib(default=None)
    SnapshotId = attrib(default=None)
    Tags = attrib(default=None)
    VolumeType = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Volume(Resource):
    """A Volume for EC2.

    See Also:
        `AWS Cloud Formation documentation for Volume
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volume.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::Volume"

    Properties: VolumeProperties = attrib(
        factory=VolumeProperties,
        converter=create_object_converter(VolumeProperties),
    )


@attrs(**ATTRSCONFIG)
class VolumeAttachmentProperties(ResourceProperties):
    Device = attrib(default=None)
    InstanceId = attrib(default=None)
    VolumeId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class VolumeAttachment(Resource):
    """A Volume Attachment for EC2.

    See Also:
        `AWS Cloud Formation documentation for VolumeAttachment
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volumeattachment.html>`_
    """

    RESOURCE_TYPE = "AWS::EC2::VolumeAttachment"

    Properties: VolumeAttachmentProperties = attrib(
        factory=VolumeAttachmentProperties,
        converter=create_object_converter(VolumeAttachmentProperties),
    )



"""General-use classes to interact with the Athena service through CloudFormation.

See Also:
    `AWS developer guide for Athena
    <https://docs.aws.amazon.com/athena/latest/ug/what-is.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import athena as _raw

# noinspection PyUnresolvedReferences
from .._raw.athena import *



"""Raw representations of every data type in the AWS Cloud9 service.

See Also:
    `AWS developer guide for Cloud9
    <https://docs.aws.amazon.com/cloud9/latest/user-guide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "EnvironmentEC2",
    "EnvironmentEC2Properties",
]


@attrs(**ATTRSCONFIG)
class EnvironmentEC2Properties(ResourceProperties):
    AutomaticStopTimeMinutes = attrib(default=None)
    Description = attrib(default=None)
    InstanceType = attrib(default=None)
    Name = attrib(default=None)
    OwnerArn = attrib(default=None)
    Repositories = attrib(default=None)
    SubnetId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class EnvironmentEC2(Resource):
    """A Environment Ec2 for Cloud9.

    See Also:
        `AWS Cloud Formation documentation for EnvironmentEC2
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloud9-environmentec2.html>`_
    """

    RESOURCE_TYPE = "AWS::Cloud9::EnvironmentEC2"

    Properties: EnvironmentEC2Properties = attrib(
        factory=EnvironmentEC2Properties,
        converter=create_object_converter(EnvironmentEC2Properties),
    )



"""Raw representations of every data type in the AWS SNS service.

See Also:
    `AWS developer guide for SNS
    <https://docs.aws.amazon.com/sns/latest/dg/welcome.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Subscription",
    "SubscriptionProperties",
    "Topic",
    "TopicProperties",
    "TopicPolicy",
    "TopicPolicyProperties",
]


@attrs(**ATTRSCONFIG)
class SubscriptionProperties(ResourceProperties):
    DeliveryPolicy = attrib(default=None)
    Endpoint = attrib(default=None)
    FilterPolicy = attrib(default=None)
    Protocol = attrib(default=None)
    RawMessageDelivery = attrib(default=None)
    Region = attrib(default=None)
    TopicArn = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Subscription(Resource):
    """A Subscription for SNS.

    See Also:
        `AWS Cloud Formation documentation for Subscription
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sns-subscription.html>`_
    """

    RESOURCE_TYPE = "AWS::SNS::Subscription"

    Properties: SubscriptionProperties = attrib(
        factory=SubscriptionProperties,
        converter=create_object_converter(SubscriptionProperties),
    )


@attrs(**ATTRSCONFIG)
class TopicProperties(ResourceProperties):
    DisplayName = attrib(default=None)
    KmsMasterKeyId = attrib(default=None)
    Subscription = attrib(default=None)
    TopicName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Topic(Resource):
    """A Topic for SNS.

    See Also:
        `AWS Cloud Formation documentation for Topic
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-topic.html>`_
    """

    RESOURCE_TYPE = "AWS::SNS::Topic"

    Properties: TopicProperties = attrib(
        factory=TopicProperties,
        converter=create_object_converter(TopicProperties),
    )


@attrs(**ATTRSCONFIG)
class TopicPolicyProperties(ResourceProperties):
    PolicyDocument = attrib(default=None)
    Topics = attrib(default=None)


@attrs(**ATTRSCONFIG)
class TopicPolicy(Resource):
    """A Topic Policy for SNS.

    See Also:
        `AWS Cloud Formation documentation for TopicPolicy
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-policy.html>`_
    """

    RESOURCE_TYPE = "AWS::SNS::TopicPolicy"

    Properties: TopicPolicyProperties = attrib(
        factory=TopicPolicyProperties,
        converter=create_object_converter(TopicPolicyProperties),
    )



"""Raw representations of every data type in the AWS KMS service.

See Also:
    `AWS developer guide for KMS
    <https://docs.aws.amazon.com/kms/latest/developerguide/overview.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Alias",
    "AliasProperties",
    "Key",
    "KeyProperties",
]


@attrs(**ATTRSCONFIG)
class AliasProperties(ResourceProperties):
    AliasName = attrib(default=None)
    TargetKeyId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Alias(Resource):
    """A Alias for KMS.

    See Also:
        `AWS Cloud Formation documentation for Alias
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-alias.html>`_
    """

    RESOURCE_TYPE = "AWS::KMS::Alias"

    Properties: AliasProperties = attrib(
        factory=AliasProperties,
        converter=create_object_converter(AliasProperties),
    )


@attrs(**ATTRSCONFIG)
class KeyProperties(ResourceProperties):
    Description = attrib(default=None)
    Enabled = attrib(default=None)
    EnableKeyRotation = attrib(default=None)
    KeyPolicy = attrib(default=None)
    KeyUsage = attrib(default=None)
    PendingWindowInDays = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Key(Resource):
    """A Key for KMS.

    See Also:
        `AWS Cloud Formation documentation for Key
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-key.html>`_
    """

    RESOURCE_TYPE = "AWS::KMS::Key"

    Properties: KeyProperties = attrib(
        factory=KeyProperties,
        converter=create_object_converter(KeyProperties),
    )



"""Raw representations of every data type in the AWS AppSync service.

See Also:
    `AWS developer guide for AppSync
    <https://docs.aws.amazon.com/appsync/latest/devguide/welcome.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "ApiKey",
    "ApiKeyProperties",
    "DataSource",
    "DataSourceProperties",
    "FunctionConfiguration",
    "FunctionConfigurationProperties",
    "GraphQLApi",
    "GraphQLApiProperties",
    "GraphQLSchema",
    "GraphQLSchemaProperties",
    "Resolver",
    "ResolverProperties",
]


@attrs(**ATTRSCONFIG)
class ApiKeyProperties(ResourceProperties):
    ApiId = attrib(default=None)
    Description = attrib(default=None)
    Expires = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ApiKey(Resource):
    """A Api Key for AppSync.

    See Also:
        `AWS Cloud Formation documentation for ApiKey
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appsync-apikey.html>`_
    """

    RESOURCE_TYPE = "AWS::AppSync::ApiKey"

    Properties: ApiKeyProperties = attrib(
        factory=ApiKeyProperties,
        converter=create_object_converter(ApiKeyProperties),
    )


@attrs(**ATTRSCONFIG)
class DataSourceProperties(ResourceProperties):
    ApiId = attrib(default=None)
    Description = attrib(default=None)
    DynamoDBConfig = attrib(default=None)
    ElasticsearchConfig = attrib(default=None)
    HttpConfig = attrib(default=None)
    LambdaConfig = attrib(default=None)
    Name = attrib(default=None)
    RelationalDatabaseConfig = attrib(default=None)
    ServiceRoleArn = attrib(default=None)
    Type = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DataSource(Resource):
    """A Data Source for AppSync.

    See Also:
        `AWS Cloud Formation documentation for DataSource
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appsync-datasource.html>`_
    """

    RESOURCE_TYPE = "AWS::AppSync::DataSource"

    Properties: DataSourceProperties = attrib(
        factory=DataSourceProperties,
        converter=create_object_converter(DataSourceProperties),
    )


@attrs(**ATTRSCONFIG)
class FunctionConfigurationProperties(ResourceProperties):
    ApiId = attrib(default=None)
    DataSourceName = attrib(default=None)
    Description = attrib(default=None)
    FunctionVersion = attrib(default=None)
    Name = attrib(default=None)
    RequestMappingTemplate = attrib(default=None)
    RequestMappingTemplateS3Location = attrib(default=None)
    ResponseMappingTemplate = attrib(default=None)
    ResponseMappingTemplateS3Location = attrib(default=None)


@attrs(**ATTRSCONFIG)
class FunctionConfiguration(Resource):
    """A Function Configuration for AppSync.

    See Also:
        `AWS Cloud Formation documentation for FunctionConfiguration
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appsync-functionconfiguration.html>`_
    """

    RESOURCE_TYPE = "AWS::AppSync::FunctionConfiguration"

    Properties: FunctionConfigurationProperties = attrib(
        factory=FunctionConfigurationProperties,
        converter=create_object_converter(FunctionConfigurationProperties),
    )


@attrs(**ATTRSCONFIG)
class GraphQLApiProperties(ResourceProperties):
    AuthenticationType = attrib(default=None)
    LogConfig = attrib(default=None)
    Name = attrib(default=None)
    OpenIDConnectConfig = attrib(default=None)
    UserPoolConfig = attrib(default=None)


@attrs(**ATTRSCONFIG)
class GraphQLApi(Resource):
    """A Graph Ql Api for AppSync.

    See Also:
        `AWS Cloud Formation documentation for GraphQLApi
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appsync-graphqlapi.html>`_
    """

    RESOURCE_TYPE = "AWS::AppSync::GraphQLApi"

    Properties: GraphQLApiProperties = attrib(
        factory=GraphQLApiProperties,
        converter=create_object_converter(GraphQLApiProperties),
    )


@attrs(**ATTRSCONFIG)
class GraphQLSchemaProperties(ResourceProperties):
    ApiId = attrib(default=None)
    Definition = attrib(default=None)
    DefinitionS3Location = attrib(default=None)


@attrs(**ATTRSCONFIG)
class GraphQLSchema(Resource):
    """A Graph Ql Schema for AppSync.

    See Also:
        `AWS Cloud Formation documentation for GraphQLSchema
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appsync-graphqlschema.html>`_
    """

    RESOURCE_TYPE = "AWS::AppSync::GraphQLSchema"

    Properties: GraphQLSchemaProperties = attrib(
        factory=GraphQLSchemaProperties,
        converter=create_object_converter(GraphQLSchemaProperties),
    )


@attrs(**ATTRSCONFIG)
class ResolverProperties(ResourceProperties):
    ApiId = attrib(default=None)
    DataSourceName = attrib(default=None)
    FieldName = attrib(default=None)
    Kind = attrib(default=None)
    PipelineConfig = attrib(default=None)
    RequestMappingTemplate = attrib(default=None)
    RequestMappingTemplateS3Location = attrib(default=None)
    ResponseMappingTemplate = attrib(default=None)
    ResponseMappingTemplateS3Location = attrib(default=None)
    TypeName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Resolver(Resource):
    """A Resolver for AppSync.

    See Also:
        `AWS Cloud Formation documentation for Resolver
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appsync-resolver.html>`_
    """

    RESOURCE_TYPE = "AWS::AppSync::Resolver"

    Properties: ResolverProperties = attrib(
        factory=ResolverProperties,
        converter=create_object_converter(ResolverProperties),
    )



"""Raw representations of every data type in the AWS Config service.

See Also:
    `AWS developer guide for Config
    <https://docs.aws.amazon.com/config/latest/developerguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "AggregationAuthorization",
    "AggregationAuthorizationProperties",
    "ConfigRule",
    "ConfigRuleProperties",
    "ConfigurationAggregator",
    "ConfigurationAggregatorProperties",
    "ConfigurationRecorder",
    "ConfigurationRecorderProperties",
    "DeliveryChannel",
    "DeliveryChannelProperties",
]


@attrs(**ATTRSCONFIG)
class AggregationAuthorizationProperties(ResourceProperties):
    AuthorizedAccountId = attrib(default=None)
    AuthorizedAwsRegion = attrib(default=None)


@attrs(**ATTRSCONFIG)
class AggregationAuthorization(Resource):
    """A Aggregation Authorization for Config.

    See Also:
        `AWS Cloud Formation documentation for AggregationAuthorization
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-aggregationauthorization.html>`_
    """

    RESOURCE_TYPE = "AWS::Config::AggregationAuthorization"

    Properties: AggregationAuthorizationProperties = attrib(
        factory=AggregationAuthorizationProperties,
        converter=create_object_converter(AggregationAuthorizationProperties),
    )


@attrs(**ATTRSCONFIG)
class ConfigRuleProperties(ResourceProperties):
    ConfigRuleName = attrib(default=None)
    Description = attrib(default=None)
    InputParameters = attrib(default=None)
    MaximumExecutionFrequency = attrib(default=None)
    Scope = attrib(default=None)
    Source = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ConfigRule(Resource):
    """A Config Rule for Config.

    See Also:
        `AWS Cloud Formation documentation for ConfigRule
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configrule.html>`_
    """

    RESOURCE_TYPE = "AWS::Config::ConfigRule"

    Properties: ConfigRuleProperties = attrib(
        factory=ConfigRuleProperties,
        converter=create_object_converter(ConfigRuleProperties),
    )


@attrs(**ATTRSCONFIG)
class ConfigurationAggregatorProperties(ResourceProperties):
    AccountAggregationSources = attrib(default=None)
    ConfigurationAggregatorName = attrib(default=None)
    OrganizationAggregationSource = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ConfigurationAggregator(Resource):
    """A Configuration Aggregator for Config.

    See Also:
        `AWS Cloud Formation documentation for ConfigurationAggregator
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configurationaggregator.html>`_
    """

    RESOURCE_TYPE = "AWS::Config::ConfigurationAggregator"

    Properties: ConfigurationAggregatorProperties = attrib(
        factory=ConfigurationAggregatorProperties,
        converter=create_object_converter(ConfigurationAggregatorProperties),
    )


@attrs(**ATTRSCONFIG)
class ConfigurationRecorderProperties(ResourceProperties):
    Name = attrib(default=None)
    RecordingGroup = attrib(default=None)
    RoleARN = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ConfigurationRecorder(Resource):
    """A Configuration Recorder for Config.

    See Also:
        `AWS Cloud Formation documentation for ConfigurationRecorder
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configurationrecorder.html>`_
    """

    RESOURCE_TYPE = "AWS::Config::ConfigurationRecorder"

    Properties: ConfigurationRecorderProperties = attrib(
        factory=ConfigurationRecorderProperties,
        converter=create_object_converter(ConfigurationRecorderProperties),
    )


@attrs(**ATTRSCONFIG)
class DeliveryChannelProperties(ResourceProperties):
    ConfigSnapshotDeliveryProperties = attrib(default=None)
    Name = attrib(default=None)
    S3BucketName = attrib(default=None)
    S3KeyPrefix = attrib(default=None)
    SnsTopicARN = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DeliveryChannel(Resource):
    """A Delivery Channel for Config.

    See Also:
        `AWS Cloud Formation documentation for DeliveryChannel
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-deliverychannel.html>`_
    """

    RESOURCE_TYPE = "AWS::Config::DeliveryChannel"

    Properties: DeliveryChannelProperties = attrib(
        factory=DeliveryChannelProperties,
        converter=create_object_converter(DeliveryChannelProperties),
    )



"""Raw representations of every data type in the AWS SageMaker service.

See Also:
    `AWS developer guide for SageMaker
    <https://docs.aws.amazon.com/sagemaker/latest/dg/whatis.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Endpoint",
    "EndpointProperties",
    "EndpointConfig",
    "EndpointConfigProperties",
    "Model",
    "ModelProperties",
    "NotebookInstance",
    "NotebookInstanceProperties",
    "NotebookInstanceLifecycleConfig",
    "NotebookInstanceLifecycleConfigProperties",
]


@attrs(**ATTRSCONFIG)
class EndpointProperties(ResourceProperties):
    EndpointConfigName = attrib(default=None)
    EndpointName = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Endpoint(Resource):
    """A Endpoint for SageMaker.

    See Also:
        `AWS Cloud Formation documentation for Endpoint
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sagemaker-endpoint.html>`_
    """

    RESOURCE_TYPE = "AWS::SageMaker::Endpoint"

    Properties: EndpointProperties = attrib(
        factory=EndpointProperties,
        converter=create_object_converter(EndpointProperties),
    )


@attrs(**ATTRSCONFIG)
class EndpointConfigProperties(ResourceProperties):
    EndpointConfigName = attrib(default=None)
    KmsKeyId = attrib(default=None)
    ProductionVariants = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class EndpointConfig(Resource):
    """A Endpoint Config for SageMaker.

    See Also:
        `AWS Cloud Formation documentation for EndpointConfig
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sagemaker-endpointconfig.html>`_
    """

    RESOURCE_TYPE = "AWS::SageMaker::EndpointConfig"

    Properties: EndpointConfigProperties = attrib(
        factory=EndpointConfigProperties,
        converter=create_object_converter(EndpointConfigProperties),
    )


@attrs(**ATTRSCONFIG)
class ModelProperties(ResourceProperties):
    Containers = attrib(default=None)
    ExecutionRoleArn = attrib(default=None)
    ModelName = attrib(default=None)
    PrimaryContainer = attrib(default=None)
    Tags = attrib(default=None)
    VpcConfig = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Model(Resource):
    """A Model for SageMaker.

    See Also:
        `AWS Cloud Formation documentation for Model
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sagemaker-model.html>`_
    """

    RESOURCE_TYPE = "AWS::SageMaker::Model"

    Properties: ModelProperties = attrib(
        factory=ModelProperties,
        converter=create_object_converter(ModelProperties),
    )


@attrs(**ATTRSCONFIG)
class NotebookInstanceProperties(ResourceProperties):
    DirectInternetAccess = attrib(default=None)
    InstanceType = attrib(default=None)
    KmsKeyId = attrib(default=None)
    LifecycleConfigName = attrib(default=None)
    NotebookInstanceName = attrib(default=None)
    RoleArn = attrib(default=None)
    SecurityGroupIds = attrib(default=None)
    SubnetId = attrib(default=None)
    Tags = attrib(default=None)
    VolumeSizeInGB = attrib(default=None)


@attrs(**ATTRSCONFIG)
class NotebookInstance(Resource):
    """A Notebook Instance for SageMaker.

    See Also:
        `AWS Cloud Formation documentation for NotebookInstance
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sagemaker-notebookinstance.html>`_
    """

    RESOURCE_TYPE = "AWS::SageMaker::NotebookInstance"

    Properties: NotebookInstanceProperties = attrib(
        factory=NotebookInstanceProperties,
        converter=create_object_converter(NotebookInstanceProperties),
    )


@attrs(**ATTRSCONFIG)
class NotebookInstanceLifecycleConfigProperties(ResourceProperties):
    NotebookInstanceLifecycleConfigName = attrib(default=None)
    OnCreate = attrib(default=None)
    OnStart = attrib(default=None)


@attrs(**ATTRSCONFIG)
class NotebookInstanceLifecycleConfig(Resource):
    """A Notebook Instance Lifecycle Config for SageMaker.

    See Also:
        `AWS Cloud Formation documentation for NotebookInstanceLifecycleConfig
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sagemaker-notebookinstancelifecycleconfig.html>`_
    """

    RESOURCE_TYPE = "AWS::SageMaker::NotebookInstanceLifecycleConfig"

    Properties: NotebookInstanceLifecycleConfigProperties = attrib(
        factory=NotebookInstanceLifecycleConfigProperties,
        converter=create_object_converter(NotebookInstanceLifecycleConfigProperties),
    )



"""Raw representations of every data type in the AWS CloudWatch service.

See Also:
    `AWS developer guide for CloudWatch
    <http://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/WhatIsCloudWatch.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Alarm",
    "AlarmProperties",
    "Dashboard",
    "DashboardProperties",
]


@attrs(**ATTRSCONFIG)
class AlarmProperties(ResourceProperties):
    ActionsEnabled = attrib(default=None)
    AlarmActions = attrib(default=None)
    AlarmDescription = attrib(default=None)
    AlarmName = attrib(default=None)
    ComparisonOperator = attrib(default=None)
    DatapointsToAlarm = attrib(default=None)
    Dimensions = attrib(default=None)
    EvaluateLowSampleCountPercentile = attrib(default=None)
    EvaluationPeriods = attrib(default=None)
    ExtendedStatistic = attrib(default=None)
    InsufficientDataActions = attrib(default=None)
    MetricName = attrib(default=None)
    Metrics = attrib(default=None)
    Namespace = attrib(default=None)
    OKActions = attrib(default=None)
    Period = attrib(default=None)
    Statistic = attrib(default=None)
    Threshold = attrib(default=None)
    TreatMissingData = attrib(default=None)
    Unit = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Alarm(Resource):
    """A Alarm for CloudWatch.

    See Also:
        `AWS Cloud Formation documentation for Alarm
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html>`_
    """

    RESOURCE_TYPE = "AWS::CloudWatch::Alarm"

    Properties: AlarmProperties = attrib(
        factory=AlarmProperties,
        converter=create_object_converter(AlarmProperties),
    )


@attrs(**ATTRSCONFIG)
class DashboardProperties(ResourceProperties):
    DashboardBody = attrib(default=None)
    DashboardName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Dashboard(Resource):
    """A Dashboard for CloudWatch.

    See Also:
        `AWS Cloud Formation documentation for Dashboard
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudwatch-dashboard.html>`_
    """

    RESOURCE_TYPE = "AWS::CloudWatch::Dashboard"

    Properties: DashboardProperties = attrib(
        factory=DashboardProperties,
        converter=create_object_converter(DashboardProperties),
    )



"""General-use classes to interact with the SDB service through CloudFormation.

See Also:
    `AWS developer guide for SDB
    <https://docs.aws.amazon.com/AmazonSimpleDB/latest/DeveloperGuide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import sdb as _raw

# noinspection PyUnresolvedReferences
from .._raw.sdb import *



"""General-use classes to interact with the FSx service through CloudFormation.

See Also:
    `AWS developer guide for FSx
    <https://docs.aws.amazon.com/fsx/latest/WindowsGuide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import fsx as _raw

# noinspection PyUnresolvedReferences
from .._raw.fsx import *
